//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyScene.h"


MyScene::MyScene()
{
	_root = new osg::Group();
	_skyBox = new SkyBox();
	_sceneNode = new osg::Group();

	_ld = osg::Vec4(1, 1, 1, 1);
	_la = osg::Vec4(0.7, 0.7, 0.7, 1);
	_lp = osg::Vec4(-1, -1, 1, 0);

	if (!osgDB::Registry::instance()->getOptions())
		osgDB::Registry::instance()->setOptions(new osgDB::Options());
	osgDB::Registry::instance()->getOptions()->setObjectCacheHint((osgDB::Options::CacheHintOptions)(osgDB::Options::CACHE_ALL));

	_options = new osgDB::ReaderWriter::Options();
	//_options->setDataVariance(osgDB::Options::STATIC);
	_options->setObjectCacheHint( osgDB::Options::CACHE_ALL );
}


//vytvori svetlo v scene... je ho mozne donastavit v podedenych treidach cez premenne _lp, _la, _ld
osg::Node* MyScene::createLight()
{
	osg::Group* lightGroup = new osg::Group;

	osg::Light* sunLight = new osg::Light;
	sunLight->setLightNum(0);
	sunLight->setPosition(_lp);
	sunLight->setAmbient(_la);
	sunLight->setDiffuse(_ld);

	osg::LightSource* lightS2 = new osg::LightSource;	
	lightS2->setLight(sunLight);
	lightS2->setLocalStateSetModes(osg::StateAttribute::ON); 

	//mt->addChild(lightS2);

	lightGroup->addChild(lightS2);

	return lightGroup;
}


//vytvori skybox
void MyScene::CreateSkyBox( std::string in_posX, std::string in_negX, std::string in_posY, std::string in_negY, std::string in_posZ, std::string in_negZ )
{
	osg::ref_ptr<osg::Geode> geode = new osg::Geode;
	geode->addDrawable( new osg::ShapeDrawable( new osg::Sphere(osg::Vec3(), 1000)) );
	geode->setCullingActive( false );
    
		
	_skyBox.get()->getOrCreateStateSet()->setTextureAttributeAndModes( 0, new osg::TexGen );
	_skyBox.get()->setEnvironmentMap( 0,
		osgDB::readImageFile(in_posX), osgDB::readImageFile(in_negX),
		osgDB::readImageFile(in_posY), osgDB::readImageFile(in_negY),
		osgDB::readImageFile(in_posZ), osgDB::readImageFile(in_negZ) );
	_skyBox.get()->addChild( geode.get() );
}


//zkompletizuje scenu a vrati root uzol, volana je iba raz v aplikacii
osg::Group* MyScene::GetScene()
{

	osg::Group *g = (osg::Group*)createLight();
	g->addChild( _sceneNode );
	g->addChild( _skyBox );	
	_root->addChild(g);
	//osgUtil::Optimizer* my_opt = new osgUtil::Optimizer();
	//my_opt->optimize(_root, osgUtil::Optimizer::ALL_OPTIMIZATIONS);
	return _root;
}


//podla cesty vytvory a umiestni do sceny object
osg::PositionAttitudeTransform* MyScene::LoadAndSetObject(std::string in_path, osg::Vec3 in_pos)
{
	return LoadAndSetObject(in_path, in_pos, 0);
}


//podla cesty vytvory a umiestni do sceny object, aj ho natoci
osg::PositionAttitudeTransform* MyScene::LoadAndSetObject(std::string in_path, osg::Vec3 in_pos, float in_angel)
{
	return LoadAndSetObject(in_path, in_pos, in_angel, osg::Vec3(1, 1, 1));
}


//podla cesty vytvory a umiestni do sceny object, aj ho natoci a zmeni velkost
osg::PositionAttitudeTransform* MyScene::LoadAndSetObject(std::string in_path, osg::Vec3 in_pos, float in_angel, osg::Vec3 in_scale)
{
	return SetObject(osgDB::readNodeFile(in_path, _options), in_pos, in_angel, in_scale);
}


//podla nodu vytvory a umiestni do sceny object
osg::PositionAttitudeTransform* MyScene::SetObject(osg::Node* in_node, osg::Vec3 in_pos, float in_angel, osg::Vec3 in_scale)
{
	osg::PositionAttitudeTransform *pat = new osg::PositionAttitudeTransform();
	pat->setScale(in_scale);
	pat->setAttitude(osg::Quat(osg::DegreesToRadians(in_angel), osg::Vec3(0,0,1)));	
	pat->setPosition(in_pos);
	pat->addChild(in_node);
	return pat;
}


osg::Vec3 MyScene::GetStartPosition()
{
	return _startPosition;
}

void MyScene::SetStartPosition(osg::Vec3 in_pos)
{
	_startPosition = in_pos;
}

osg::Vec4 MyScene::GetClearColor()
{
	return osg::Vec4(0, 0, 0, 0);
}

void MyScene::CreateScene(bool in_paged, bool in_moreTextures, bool in_mipmap, bool in_impostor)
{
}

