//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include <osg/Geode>
#include <osg/TexGen>
#include <osg/Texture2D>

#include <osgDB/ReadFile>
#include <osgDB/WriteFile>

#include <osg/ShapeDrawable>
#include <osg/MatrixTransform>
#include <osgViewer/Viewer>
#include <osgViewer/ViewerEventHandlers>
#include <osgGA/StateSetManipulator>

#include <string>

#include "MyFirstPersonManipulator.h"

#include "MyKeyboardHandler.h"

#include "MyTerrainScene.h"
#include "MyFullScene.h"
#include "MySpaceScene.h"

#include <time.h>



// vytvori okno 
osgViewer::Viewer* CreateViewer(int in_x, int in_y, int in_w, int in_h)
{
	osgViewer::Viewer* viewer = new osgViewer::Viewer();

	osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
	traits->x = in_x;
	traits->y = in_y;
	traits->width = in_w;
	traits->height = in_h;
	traits->windowDecoration = true;
	traits->doubleBuffer = true;
	traits->sharedContext = 0;
	traits->supportsResize = false;

	osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext( traits.get() );
	viewer->getCamera()->setGraphicsContext( gc.get() );
	viewer->getCamera()->setViewport( new osg::Viewport( 0, 0, traits->width, traits->height ) );
	viewer->getCamera()->setProjectionMatrixAsPerspective(30.0f, static_cast<double>(traits->width)/static_cast<double>(traits->height), 1, 10000.0);
	
	//statistiky
	viewer->addEventHandler(new osgViewer::StatsHandler);
	//wireframe
	viewer->addEventHandler( new osgGA::StateSetManipulator(viewer->getCamera()->getOrCreateStateSet()) );

	return viewer;
}


int main(int argc, char **argv)
{
	bool pp = false, pt = false, pm = false, pi = false;
	int scene_int = 1;
	std::string param;
	for (int t=1; t<argc; t++) {
		param = std::string(argv[t]);
		if (param == "-p")
			pp = true;
		else if (param == "-t")
			pt = true;
		else if (param == "-m")
			pm = true;
		else if (param == "-i")
			pi = true;
		else if (param == "-scene1")
			scene_int = 1;
		else if (param == "-scene2")
			scene_int = 2;
		else if (param == "-scene3")
			scene_int = 3;
	}

	double timeelapsed = 0.0;
	double lastTime = 0.0;
	double step = 5.0;

	const clock_t start = clock();

	MyScene *scene;
	MyFirstPersonManipulator* fpm = new MyFirstPersonManipulator();
	MyKeyboardHandler* kh = new MyKeyboardHandler();


	osgViewer::Viewer* viewer = CreateViewer(100, 100, 800, 600);
	viewer->setCameraManipulator(fpm);
	
	viewer->addEventHandler(kh);


	//vytvorenie konkretnej sceny
	if (scene_int == 1)
		scene = new MyTerrainScene();
	else if (scene_int == 3)
		scene = new MySpaceScene();
	else
		scene = new MyFullScene();

	scene->CreateScene(pp, pt, pm, pi);


	osg::Vec3d a, b, c;
	((MyFirstPersonManipulator*)viewer->getCameraManipulator())->getHomePosition(a, b, c);
	((MyFirstPersonManipulator*)viewer->getCameraManipulator())->setHomePosition(scene->GetStartPosition(), b, c);

	viewer->getCamera()->setClearColor(scene->GetClearColor());
	
	
	viewer->setSceneData( scene->GetScene() );
	

    viewer->realize();
	
	int countFrame = 0;
	time_t startTime;
	bool b_write_time = false;
    while(!viewer->done())
    {
        viewer->frame();	
		countFrame++;

		if (!b_write_time) {
			//pocitanie casu vytvarania sceny - cas do prveho renderu
			std::cout << "Created scene in " << (float)(clock() - start) /  (float)CLOCKS_PER_SEC << "\n";
			b_write_time = true;
			startTime = time(NULL); 
		}

		//vypocet casu pre pohyb
		timeelapsed = viewer->elapsedTime() - lastTime;
		lastTime = viewer->elapsedTime();

		fpm->SetTimeElapsed(timeelapsed);

		
		if (kh->SHIFT)
			step = 100.0;
		else 
			step = 15.0;

		if (kh->W)
			fpm->goForward(step);
		if (kh->D)
			fpm->goRight(step);
		if (kh->A)
			fpm->goRight(-step);
		if (kh->S)
			fpm->goForward(-step);
    }
    //vypisanie priemerneho fps
	std::cout << "Average FPS: " <<  (double)countFrame/(time(NULL) - startTime); 

	return 0;
}
