//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyCapacityCounter.h"


#define IN_STACK				2	//00000010
#define OUT_STACK				253	//11111101
#define FAIL					1	//00000001

#define ALL_COMPLETE			252 //11111100

#define X_PLUS_COMPLETE			128	//10000000
#define X_MINUS_COMPLETE		64	//01000000

#define Y_PLUS_COMPLETE			32	//00100000
#define Y_MINUS_COMPLETE		16	//00010000

#define Z_PLUS_COMPLETE			8	//00001000
#define Z_MINUS_COMPLETE		4	//00000100


MyCapacityCounter::MyCapacityCounter()
{


	data = new unsigned char**[GRID_SIZE];
	for (int i = 0; i < GRID_SIZE; ++i) {
		data[i] = new unsigned char*[GRID_SIZE];

		for (int j = 0; j < GRID_SIZE; ++j)
			data[i][j] = new unsigned char[GRID_SIZE];
	}

	for (int x=0; x<GRID_SIZE; x++)
		for (int y=0; y<GRID_SIZE; y++)
			for (int z=0; z<GRID_SIZE; z++)
				data[x][y][z] = 0;

	_poc_lines = 0;
	for (int t=0; t<32; t++)
		_lines[t] = new osg::LineSegment();

	one_step = osg::Vec3(1,1,1);
	vec_max = osg::Vec3(0,0,0);
	vec_min = osg::Vec3(0,0,0);

	stackX = new std::vector<unsigned char>();
	stackY = new std::vector<unsigned char>();
	stackZ = new std::vector<unsigned char>();

	//iv = new osgUtil::IntersectVisitor();
	
}

MyCapacityCounter::~MyCapacityCounter()
{
	for (int i = 0; i < GRID_SIZE; ++i) {
		for (int j = 0; j < GRID_SIZE; ++j)
			delete [] data[i][j];

		delete [] data[i];
	}
	delete [] data;
	delete stackX;
	delete stackY;
	delete stackZ;
}


double MyCapacityCounter::GetCapacity(Node* in_node)
{
	MyBoundingBox bb;
	_object = in_node;

	ClearStack();
	in_node->accept(bb);
	vec_max = bb.Max;
	vec_min = bb.Min;
	//pridanie kusok na okrajoch aby urcite bol okolo objektu kvader ktory sa ho nedotyka
	// -2 lebo najprv vypocitame velkost mriezky pre mensiu mriezku a potom okolo boundingboxu pridame jednu radu do kazdej strany (v jednej osy 2 rady)
	one_step = osg::Vec3(vec_max - vec_min) / (float)(GRID_SIZE-2);
	//pridanie po okrajoch rady - zvetsenie boundingboxu
	vec_min -= one_step;
	vec_max += one_step;
	
	//zacneme ratat objem, pridanim prveho bodu 0,0,0 do stacku
	AddPointToStack(Vec3i(0, 0, 0));
	//pokial stack neni prazdny, pracujeme na ratani
	osg::Vec3i tmp_vec;
	while (GetStackSize() > 0) {
		tmp_vec = GetPointFromStack();
		ComputePoint(tmp_vec);		
		if (GetStackSize() == 0)
			FlushRays();
	}

	//zaverecny vypocet kolko bodov bolo preichodzych, kolko fail atd
	unsigned long poc = GetMaxCapacity(), pocf = 0;
	for (int x=0; x<GRID_SIZE; x++) {
		for (int y=0; y<GRID_SIZE; y++) {
			for (int z=0; z<GRID_SIZE; z++) {
				if (IsCompleteAll(Vec3i(x, y, z)) && !IsPointValue(Vec3i(x, y, z), FAIL)) {
					poc--;
				}
				if (IsPointValue(Vec3i(x, y, z), FAIL)) {
					pocf++;
				}
			}
		}
	}
	float max = GetMaxCapacity();
	Vec3 size = vec_max - vec_min;
	return ((double)(poc - pocf/2) / max) * (double)(abs(size.x()) * abs(size.y()) * abs(size.z()));
}

unsigned long MyCapacityCounter::GetMaxCapacity()
{
	return GRID_SIZE*GRID_SIZE*GRID_SIZE;
}


bool MyCapacityCounter::IsOutOfRange(Vec3i in_xyz)
{
	if (in_xyz.x() < 0 || in_xyz.x() >= GRID_SIZE)
		return true;
	if (in_xyz.y() < 0 || in_xyz.y() >= GRID_SIZE)
		return true;
	if (in_xyz.z() < 0 || in_xyz.z() >= GRID_SIZE)
		return true;
	return false;
}

//ak je range mimo siet, nastavi na posledne mozne pozicie
Vec3i MyCapacityCounter::RepairRange(Vec3i in_xyz)
{
	float x = in_xyz.x();
	float y = in_xyz.y();
	float z = in_xyz.z();
	if (x < 0) x = 0;
	if (x >= GRID_SIZE) x = GRID_SIZE-1;
	if (y < 0) y = 0;
	if (y >= GRID_SIZE) y = GRID_SIZE-1;
	if (z < 0) z = 0;
	if (z >= GRID_SIZE) z = GRID_SIZE-1;
	return Vec3i(x, y , z);
}

bool MyCapacityCounter::IsPointValue(Vec3i in_xyz, unsigned char in_data)
{
	if (IsOutOfRange(in_xyz))
		return true;
	return (data[in_xyz.x()][in_xyz.y()][in_xyz.z()] & in_data) != 0;
}

void MyCapacityCounter::SetPointValue(Vec3i in_xyz, unsigned char in_data)
{
	if (IsOutOfRange(in_xyz))
		return;
	data[in_xyz.x()][in_xyz.y()][in_xyz.z()] |= in_data;
}

bool MyCapacityCounter::IsCompleteAll(Vec3i in_xyz)
{
	if (IsOutOfRange(in_xyz))
		return true;
	return ((data[in_xyz.x()][in_xyz.y()][in_xyz.z()] & ALL_COMPLETE) == ALL_COMPLETE);
}


//spracovanie bodu zo stacku
void MyCapacityCounter::ComputePoint(Vec3i in_vec)
{

	if (IsPointValue(in_vec, FAIL))
		return;

	if (IsCompleteAll(in_vec))
		return;

	if (!IsPointValue(in_vec, X_PLUS_COMPLETE)) {
		if (in_vec.x() < GRID_SIZE - 1) 
			ThrowRay(in_vec, Vec3i(1, 0, 0));
		SetPointValue(in_vec, X_PLUS_COMPLETE);
	}

	if (!IsPointValue(in_vec, X_MINUS_COMPLETE)) {
		if (in_vec.x() > 0) 
			ThrowRay(in_vec, Vec3i(-1, 0, 0));
		SetPointValue(in_vec, X_MINUS_COMPLETE);
	}



	if (!IsPointValue(in_vec, Y_PLUS_COMPLETE)) {
		if (in_vec.y() < GRID_SIZE - 1) 
			ThrowRay(in_vec, Vec3i(0, 1, 0));
		SetPointValue(in_vec, Y_PLUS_COMPLETE);
	}

	if (!IsPointValue(in_vec, Y_MINUS_COMPLETE)) {
		if (in_vec.y() > 0) 
			ThrowRay(in_vec, Vec3i(0, -1, 0));
		SetPointValue(in_vec, Y_MINUS_COMPLETE);
	}



	if (!IsPointValue(in_vec, Z_PLUS_COMPLETE)) {
		if (in_vec.z() < GRID_SIZE - 1) 
			ThrowRay(in_vec, Vec3i(0, 0, 1));
		SetPointValue(in_vec, Z_PLUS_COMPLETE);
	}

	if (!IsPointValue(in_vec, Z_MINUS_COMPLETE)) {
		if (in_vec.z() > 0) 
			ThrowRay(in_vec, Vec3i(0, 0, -1));
		SetPointValue(in_vec, Z_MINUS_COMPLETE);
	}
}


void MyCapacityCounter::ClearStack()
{
	stackX->clear();
	stackY->clear();
	stackZ->clear();
}


int MyCapacityCounter::GetStackSize()
{
	return stackX->size();
}


void MyCapacityCounter::AddPointToStack(Vec3i in_xyz)
{
	if (IsPointValue(in_xyz, IN_STACK))
		return;
	stackX->push_back((unsigned char)in_xyz.x());
	stackY->push_back((unsigned char)in_xyz.y());
	stackZ->push_back((unsigned char)in_xyz.z());
	SetPointValue(in_xyz, IN_STACK);
}

Vec3i MyCapacityCounter::GetPointFromStack()
{
	Vec3i tmp_vec(stackX->front(), stackY->front(), stackZ->front());
	stackX->erase(stackX->begin());
	stackY->erase(stackY->begin());
	stackZ->erase(stackZ->begin());
	data[tmp_vec.x()][tmp_vec.y()][tmp_vec.z()] &= OUT_STACK;
	return tmp_vec;
}


//funkcia bude trasovat z bodu in_a do bodu in_b (podla integrovskych X,Y,Z) a nastavi body z tychto stran ako priechodzie
void MyCapacityCounter::SetCompletedByRay(osg::Vec3 in_a, osg::Vec3 in_b)
{
	//vypocitame 2 body podla ktorych budeme trasovat (integerove pozicie X,Y,Z)
	osg::Vec3i a_vec = GetXYZFromPosition(in_a);
	osg::Vec3i b_vec = GetXYZFromPosition(in_b);
	osg::Vec3i tmp_vec;
	
	//floatovy vektor dlzky - tento sa bude trasovat
	osg::Vec3 v = (in_b - in_a);

	//inicializujeme premenne pricom do X,Y,Z zaznacime orientaciu suradnic trasovaneho vektora	
	//tie potom prepisem na zacinajuci bod trasovania, ale pripadne swapneme vektory podla orientacie suradnic 	
	int x1 = v.x() < 0 ? -1 : 1;
	int y1 = v.y() < 0 ? -1 : 1;
	int z1 = v.z() < 0 ? -1 : 1;
	int x2 = 0, y2 = 0, z2 = 0; // tieto premenne sluzia na vypocet "to" for cyklu
	
	//inicializovanie premennych pre cyklus - defaultne hodnoty nastavene pre X os
	int *from = &x1, *to = &x2;					//premenne pre cyklus.. pojde od "from" do "to"
	unsigned char val_c1 = X_PLUS_COMPLETE, val_c2 = X_MINUS_COMPLETE;			// premenne do cyklu pre zmenu stavu a kompletu

	//zistime os po ktorej pojdeme - ci Y alebo Z (X je defaultna)
	if (abs(v.y()) > abs(v.x()) && abs(v.y()) > abs(v.z())) {
		from = &y1;
		to = &y2;
		val_c1 = Y_PLUS_COMPLETE;
		val_c2 = Y_MINUS_COMPLETE;
	}
	else if (abs(v.z()) > abs(v.x()) && abs(v.z()) > abs(v.y())) {
		from = &z1;
		to = &z2;
		val_c1 = Z_PLUS_COMPLETE;
		val_c2 = Z_MINUS_COMPLETE;
	}

	// nastavime inkrementaciu alebo dekrementaciu cyklu
	int dir = (*from);


	//inicializujeme X,Y,Z na zaciatok ciary
	x1 = a_vec.x();
	y1 = a_vec.y();
	z1 = a_vec.z(); 
	x2 = b_vec.x();
	y2 = b_vec.y();
	z2 = b_vec.z();

	//X,Y, alebo Z sa pripocitava cez pointer "from"
	for (;(*from)!=(*to)+dir; (*from)+=dir) {
		data[x1][y1][z1] |= val_c1 | val_c2;
		if (!IsCompleteAll(Vec3i(x1, y1, z1)))
			AddPointToStack(Vec3i(x1, y1, z1));
	}

}

// pustime bufers s rays do modelu aby sa nasli kolizie
void MyCapacityCounter::FlushRays()
{
	osgUtil::IntersectVisitor *iv = new osgUtil::IntersectVisitor();	
	for (int t=0; t<_poc_lines; t++)
		iv->addLineSegment(_lines[t]);
    _object->accept(*iv);

	osgUtil::IntersectVisitor::HitList ivhl;
	for (int t=0; t<_poc_lines; t++) {
		osg::LineSegment* line = _lines[t];
		ivhl = iv->getHitList(line);
		osgUtil::Hit hit1, hit2;
		if (ivhl.empty()) {
			SetCompletedByRay(line->start(), line->end());
		}
		else {		
			SetCompletedByRay(line->start(), ivhl.front().getWorldIntersectPoint());
			SetCompletedByRay(ivhl.back().getWorldIntersectPoint(), line->end());
			SetPointValue(GetXYZFromPosition(ivhl.front().getWorldIntersectPoint()), FAIL);
			SetPointValue(GetXYZFromPosition(ivhl.back().getWorldIntersectPoint()), FAIL);
		}
	}
	_poc_lines = 0;
}


void MyCapacityCounter::ThrowRay(osg::Vec3i in_xyz, osg::Vec3i in_vec)
{
	osg::Vec3 start = GetPositionFromXYZ(in_xyz);
	osg::Vec3 end = GetPositionFromXYZ(in_xyz + in_vec * GRID_SIZE);
	
	_lines[_poc_lines]->set(start, end);
	_poc_lines++;

	if (_poc_lines == 32)
		FlushRays();

}


osg::Vec3 MyCapacityCounter::GetPositionFromXYZ(osg::Vec3i in_xyz)
{
	return vec_min + osg::Vec3(one_step.x() * ((float)in_xyz.x()+0.5), one_step.y() * ((float)in_xyz.y()+0.5), one_step.z() * ((float)in_xyz.z()+0.5));
}


osg::Vec3i MyCapacityCounter::GetXYZFromPosition(osg::Vec3 in_pos)
{
	osg::Vec3 tmp_vec = (in_pos - vec_min);
	tmp_vec = osg::Vec3(tmp_vec.x()/one_step.x(), tmp_vec.y()/one_step.y(), tmp_vec.z()/one_step.z());
	Vec3i rtv = osg::Vec3i((int)tmp_vec.x(), (int)tmp_vec.y(), (int)tmp_vec.z());
	if (IsOutOfRange(rtv)) 
		return RepairRange(rtv);
	return rtv;
}

