//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyGUIMainWindow.h"


MyGUIMainWindow::MyGUIMainWindow() : QMainWindow()
{
	COLOR_GREY = osg::Vec4(0.6, 0.6, 0.6, 1);
	COLOR_BLACK = osg::Vec4(0, 0, 0, 1);
	COLOR_GREEN = osg::Vec4(0.2, 0.6, 0.2, 1);


	_imageCompareWindow = new MyGUIImageCompare();
	_capacityCounter = new MyCapacityCounter();

	_toImageCompare = 0;
	_toSimplify = -1;	

    setThreadingModel(osgViewer::ViewerBase::SingleThreaded);

    setKeyEventSetsDone(0);

	InitGUI();

    connect( &_guiLoopTimer, SIGNAL(timeout()), this, SLOT(update()) );
    _guiLoopTimer.start( 10 );

	setGeometry( 100, 100, 1000, 650 );
	showMaximized();
	
}


void MyGUIMainWindow::HideAllWindows()
{
	_imageCompareWindow->hide();
}


osg::Group* MyGUIMainWindow::GetViewGroup(int in_view)
{
	return (Group*)(((Group*)(getView(in_view)->getSceneData()))->getChild(1));
}


void MyGUIMainWindow::InitGUI() 
{


	InitMenu();
	
	InitToolBar();

	InitLeftBar();
		
	InitStatusBar();

	InitWidgets();	
		
	RefreshGUI();

}


void MyGUIMainWindow::GUIEnable(bool in_bool)
{
	_mainWidget->setEnabled(in_bool);
	_toolBar->setEnabled(in_bool);
	_menuBar->setEnabled(in_bool);
}


void MyGUIMainWindow::InitMenuActions() 
{
	// Project->New
	   _maProjectNew = new QAction( this );
	   _maProjectNew->setText( "New project" );	   
	   connect( _maProjectNew, SIGNAL( triggered() ), this, SLOT( OnProjectNew() ) );

	// Project->Load
	   _maProjectLoad = new QAction( this );
	   _maProjectLoad->setText( "Load project" );	   
	   connect( _maProjectLoad, SIGNAL( triggered() ), this, SLOT( OnProjectLoad() ) );

	// Project->Save
	   _maProjectSave = new QAction( this );
	   _maProjectSave->setText( "Save project" );	   
	   connect( _maProjectSave, SIGNAL( triggered() ), this, SLOT( OnProjectSave() ) );

	// Project->Export as LOD dist
	   _maProjectELODDist = new QAction( this );
	   _maProjectELODDist->setText( "Export as LOD" );	   
	   connect( _maProjectELODDist, SIGNAL( triggered() ), this, SLOT( OnProjectELOD_Distance() ) );

	// Project->export as PLOD dist 
	   _maProjectEPLODDist = new QAction( this );
	   _maProjectEPLODDist->setText( "Export as PagedLOD" );	   
	   connect( _maProjectEPLODDist, SIGNAL( triggered() ), this, SLOT( OnProjectEPLOD_Distance() ) );

	// Project->Export as LOD Pix
	   _maProjectELODPix = new QAction( this );
	   _maProjectELODPix->setText( "Export as LOD" );	   
	   connect( _maProjectELODPix, SIGNAL( triggered() ), this, SLOT( OnProjectELOD_Pixels() ) );

	// Project->export as PLOD Pix 
	   _maProjectEPLODPix = new QAction( this );
	   _maProjectEPLODPix->setText( "Export as PagedLOD" );	   
	   connect( _maProjectEPLODPix, SIGNAL( triggered() ), this, SLOT( OnProjectEPLOD_Pixels() ) );

	   

	// Project->Export as OBJ
	   _maProjectEOBJ = new QAction( this );
	   _maProjectEOBJ->setText( "Export as OBJs" );	   
	   connect( _maProjectEOBJ, SIGNAL( triggered() ), this, SLOT( OnProjectEOBJ() ) );

	// Project->Export as OBJ
	   _maProjectEImpostor = new QAction( this );
	   _maProjectEImpostor->setText( "Export as Impostor" );	   
	   connect( _maProjectEImpostor, SIGNAL( triggered() ), this, SLOT( OnProjectEImpostor() ) );
	   

	// Project->Exit
		_maProjectExit = new QAction( this );
		_maProjectExit->setText( "Exit" );	   
		connect( _maProjectExit, SIGNAL( triggered() ), qApp, SLOT( quit() ) );



	// Tools->Preview
		_maToolsPreview = new QAction( this );
		_maToolsPreview->setText( "Preview" );
		connect( _maToolsPreview, SIGNAL( triggered() ), this, SLOT( OnToolsPreview() ) );

	// Tools->Auto Convert
		_maToolsAuto = new QAction( this );
		_maToolsAuto->setText( "Auto Convert" );
		connect( _maToolsAuto, SIGNAL( triggered() ), this, SLOT( OnToolsAutoConvert() ) );

	// Tools->repair
		_maToolsRepair = new QAction( this );
		_maToolsRepair->setText( "Repair selected mesh" );
		connect( _maToolsRepair, SIGNAL( triggered() ), this, SLOT( OnRepair() ) );


	// Help->Help
		_maHelpHelp = new QAction( this );
		_maHelpHelp->setText( "Help" );

	// Help->About
		_maHelpAbout = new QAction( this );
		_maHelpAbout->setText( "About" );
}


void MyGUIMainWindow::InitLeftBar() 
{

	_leftBar = new QToolBar( this );
	_leftBar->setObjectName( "LeftBar" );
	_leftBar->setMovable( false );
	_leftBar->setOrientation( Qt::Vertical );
	addToolBar( Qt::LeftToolBarArea, _leftBar );



	_bAddLOD = new QPushButton();
	_bAddLOD->setIcon(QIcon(":/LODExporter/addButton"));
	connect(_bAddLOD, SIGNAL(clicked()), this, SLOT(OnAddLOD()));

	_bRemoveLOD = new QPushButton();
	_bRemoveLOD->setIcon(QIcon(":/LODExporter/removeButton"));
	connect(_bRemoveLOD, SIGNAL(clicked()), this, SLOT(OnRemoveLOD()));

	QHBoxLayout *hb = new QHBoxLayout();
	hb->addWidget(_bAddLOD);
	hb->addWidget(_bRemoveLOD);
	QWidget *q = new QWidget();
	q->setLayout(hb);


	_leftBar->addWidget( q );
	_lSelectLOD = new QListWidget();
	_lSelectLOD->setFixedWidth(120);
	_lSelectLOD->setSelectionMode(QListWidget::SingleSelection);
	QObject::connect(_lSelectLOD, SIGNAL(currentRowChanged(int)), this, SLOT(OnSelectLOD()));
	_leftBar->addWidget(_lSelectLOD);
}


void MyGUIMainWindow::InitMenu() 
{
	
	InitMenuActions();

	_menuBar = new QMenuBar(this);
	setMenuBar(_menuBar);

	
	_mProject = new QMenu("Project", _menuBar);
		_mExportDist = new QMenu("Export with Distance", _mProject );
		_mExportPixel = new QMenu("Export with PixelSize", _mProject );
	_mTools = new QMenu("Tools", _menuBar);
	_mHelp = new QMenu("Help", _menuBar);

	_menuBar->addMenu(_mProject);
	_menuBar->addMenu(_mTools);
	//_menuBar->addMenu(_mHelp);

	
	_mProject->addAction(_maProjectNew);
	_mProject->addAction(_maProjectLoad);
	_mProject->addAction(_maProjectSave);
	_mProject->addAction(_mExportDist->menuAction());
	_mProject->addAction(_mExportPixel->menuAction());
	_mProject->addAction(_maProjectEImpostor);
	_mProject->addAction(_maProjectEOBJ);	
	_mProject->addAction(_maProjectExit);
	
	_mExportDist->addAction(_maProjectELODDist);
	_mExportDist->addAction(_maProjectEPLODDist);
	
	_mExportPixel->addAction(_maProjectELODPix);
	_mExportPixel->addAction(_maProjectEPLODPix);

	_mTools->addAction(_maToolsAuto);
	_mTools->addAction(_maToolsPreview);
	_mTools->addAction(_maToolsRepair);
	
	_mHelp->addAction(_maHelpHelp);
	_mHelp->addAction(_maHelpAbout);
	

}


void MyGUIMainWindow::InitToolBar() 
{

	_toolBar = new QToolBar( this );
	_toolBar->setObjectName( "Toolbar" );
	_toolBar->setMovable( false );
	_toolBar->setOrientation( Qt::Horizontal );
	addToolBar( Qt::TopToolBarArea, _toolBar );
	
	_toolTabs = new QTabWidget(_toolBar);
	_toolTabMesh = new QWidget();
	_toolTabTexture = new QWidget();
	_toolTabBillboard = new QWidget();
	_toolTabCompares = new QWidget();
	_toolTabViews = new QWidget();
	

	_toolBar->addWidget(_toolTabs);
	
	_toolTabs->setFixedHeight(QStyle::PM_ToolBarIconSize);
  	_toolTabs->addTab(_toolTabMesh,"Mesh tools");  
	_toolTabs->addTab(_toolTabTexture,"Texture tools");
	_toolTabs->addTab(_toolTabBillboard,"Billboard tools");
	_toolTabs->addTab(_toolTabCompares,"Compare tools");
	_toolTabs->addTab(_toolTabViews,"View tools");

	
	QHBoxLayout *horizont; 
	
	
	
	//start mesh tab===============================

	horizont = new QHBoxLayout(_toolTabMesh);
	_toolTabMesh->setLayout(horizont);

	//Simplifier
		//combo=====
		horizont->addWidget( new QLabel( "  Select Simplifier  ", this ) );
		_cSelectSimplifier = new QComboBox();
		_cSelectSimplifier->addItems(MyLOD::GetMethodStrings());
		horizont->addWidget(_cSelectSimplifier, 0, Qt::AlignLeft);
		


		//ratio=====
		horizont->addWidget( new QLabel( "   Ratio(%)  ", this ));
		_dsRatio = new QDoubleSpinBox( this );
		_dsRatio->setMinimum(1.0);
		_dsRatio->setMaximum(99.0);
		_dsRatio->setSingleStep( 0.5 );
		_dsRatio->setValue( 90.0 );
		horizont->addWidget(_dsRatio );




		//vl====
		horizont->addWidget( new QLabel( "   Max edge error  ", this ));
		//checkbox
			_checkError = new QCheckBox(" ");
			connect(_checkError, SIGNAL(stateChanged(int)), this, SLOT(OnCheckError()));
			horizont->addWidget(_checkError );
		//============
		_dsError = new QDoubleSpinBox( this );
		_dsError->setSingleStep( 0.1 );
		_dsError->setValue( 1.0 );
		_dsError->setMaximum(1000);
		horizont->addWidget(_dsError );


		//checkbox
			horizont->addWidget( new QLabel( "   Don't simplify probleming edges  ", this ));
			_checkEdges = new QCheckBox(" ");	
			_checkEdges->setChecked(true);
			horizont->addWidget(_checkEdges );
		//============

		//simplify====
		_bSimplify = new QPushButton(this);		
		horizont->addWidget( _bSimplify );
		_bSimplify->setText("Simplify");
		_bSimplify->setIcon(QIcon(":/LODExporter/Simplify"));
		connect(_bSimplify, SIGNAL(clicked()), this, SLOT(OnSimplify()));


	

		horizont->addSpacing(20);


		//Undo=====
		_bUndo = new QPushButton(this);		
		horizont->addWidget( _bUndo );
		_bUndo->setText("Undo");
		_bUndo->setIcon(QIcon(":/LODExporter/Undo"));
		connect(_bUndo, SIGNAL(clicked()), this, SLOT(OnUndo()));

		//Redo====
		_bRedo = new QPushButton(this);		
		horizont->addWidget( _bRedo );
		_bRedo->setText("Redo");
		_bRedo->setIcon(QIcon(":/LODExporter/Redo"));
		connect(_bRedo, SIGNAL(clicked()), this, SLOT(OnRedo()));


	horizont->addStretch();
	horizont->setSpacing(0);
	///end of tab=======================================








	//start texture tab===============================
	horizont = new QHBoxLayout(_toolTabTexture);
	_toolTabTexture->setLayout(horizont);


	//Texture resizer
		horizont->addWidget( new QLabel( "  Select Texture  ", this ) );
		_cSelectTexture = new QComboBox();
		_cSelectTexture->setFixedWidth(200);
		_cSelectTexture->addItems(MyLODProject::GetProject()->GetTexturesNames());
		horizont->addWidget(_cSelectTexture);
	

		horizont->addWidget( new QLabel( "  Select Quality  ", this ) );
		_cSelectTextureQuality = new QComboBox();
		_cSelectTextureQuality->addItems(MyLODProject::GetProject()->GetTexturesQuality());
		horizont->addWidget(_cSelectTextureQuality);

		horizont->addSpacing(10);

		_bResizeTexture = new QPushButton(this);		
		horizont->addWidget( _bResizeTexture );
		_bResizeTexture->setText("Resize");
		_bResizeTexture->setIcon(QIcon(":/LODExporter/Simplify"));
		connect(_bResizeTexture, SIGNAL(clicked()), this, SLOT(OnTextureResize()));
		
	horizont->addStretch();
	horizont->setSpacing(0);
	///end of tab=======================================




	//start billboard tab===============================
	horizont = new QHBoxLayout(_toolTabBillboard);
	_toolTabBillboard->setLayout(horizont);

		//create billboard
			_bCreateBillboard = new QPushButton(this);
			horizont->addWidget( _bCreateBillboard );
			_bCreateBillboard->setText("Set as Billboard");
			connect(_bCreateBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardCreate()));

			horizont->addSpacing(5);

		//delete billboard
			_bDeleteBillboard = new QPushButton(this);
			horizont->addWidget( _bDeleteBillboard );
			_bDeleteBillboard->setText("Unset Billboard");
			connect(_bDeleteBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardDelete()));

			horizont->addSpacing(20);

		//load billboard
			_bLoadBillboard = new QPushButton(this);
			horizont->addWidget( _bLoadBillboard );
			_bLoadBillboard->setText("Load Image");
			connect(_bLoadBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardLoadImage()));

			horizont->addSpacing(5);

		//rotate billboard
			_bRotateBillboard = new QPushButton(this);
			horizont->addWidget( _bRotateBillboard );
			_bRotateBillboard->setText("Rotate");
			connect(_bRotateBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardRotate()));

			horizont->addSpacing(5);

		//flip billboard
			_bFlipBillboard = new QPushButton(this);
			horizont->addWidget( _bFlipBillboard );
			_bFlipBillboard->setText("Flip");
			connect(_bFlipBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardFlip()));

			horizont->addSpacing(10);

		//Scale billboard
			horizont->addWidget( new QLabel( " Width factor: "));
			_dsWidthBillboard = new QDoubleSpinBox( this );
			_dsWidthBillboard->setMinimum(0.1);
			_dsWidthBillboard->setMaximum(2.0);
			_dsWidthBillboard->setSingleStep( 0.1 );
			_dsWidthBillboard->setValue( 1.0 );
			horizont->addWidget(_dsWidthBillboard )	;

				horizont->addSpacing(5);

			horizont->addWidget( new QLabel( " Height factor: "));
			_dsHeightBillboard = new QDoubleSpinBox( this );
			_dsHeightBillboard->setMinimum(0.1);
			_dsHeightBillboard->setMaximum(2.0);
			_dsHeightBillboard->setSingleStep( 0.1 );
			_dsHeightBillboard->setValue( 1.0 );
			horizont->addWidget(_dsHeightBillboard );			

				horizont->addSpacing(5);

			_bScaleBillboard = new QPushButton(this);
			horizont->addWidget( _bScaleBillboard );
			_bScaleBillboard->setText("Scale Billboard");
			connect(_bScaleBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardScale()));

				horizont->addSpacing(5);
			//reset scale
			_bScaleResetBillboard = new QPushButton(this);
			horizont->addWidget( _bScaleResetBillboard );
			_bScaleResetBillboard->setText("Reset scale");
			connect(_bScaleResetBillboard, SIGNAL(clicked()), this, SLOT(OnBillboardResetScale()));

			

				horizont->addSpacing(10);
			horizont->addWidget( new QLabel( " Set Mode: "));
		//mode of billboard		
			_bModeBillboardPoint = new QPushButton(this);
			horizont->addWidget( _bModeBillboardPoint );
			_bModeBillboardPoint->setText("Point");
			connect(_bModeBillboardPoint, SIGNAL(clicked()), this, SLOT(OnBillboardSetModePoint()));

			_bModeBillboardX = new QPushButton(this);
			horizont->addWidget( _bModeBillboardX );
			_bModeBillboardX->setText("Axis X");
			connect(_bModeBillboardX, SIGNAL(clicked()), this, SLOT(OnBillboardSetModeX()));

			_bModeBillboardY = new QPushButton(this);
			horizont->addWidget( _bModeBillboardY );
			_bModeBillboardY->setText("Axis Y");
			connect(_bModeBillboardY, SIGNAL(clicked()), this, SLOT(OnBillboardSetModeY()));

			_bModeBillboardZ = new QPushButton(this);
			horizont->addWidget( _bModeBillboardZ );
			_bModeBillboardZ->setText("Axis Z");
			connect(_bModeBillboardZ, SIGNAL(clicked()), this, SLOT(OnBillboardSetModeZ()));

	horizont->addStretch();
	horizont->setSpacing(0);
	///end of tab=======================================




	//start compare tab===============================
	horizont = new QHBoxLayout(_toolTabCompares);
	_toolTabCompares->setLayout(horizont);


	//Image Compare
		_bICompareI = new QPushButton(this);
		horizont->addWidget( _bICompareI );
		_bICompareI->setText("Images Compare");
		_bICompareI->setIcon(QIcon(":/LODExporter/CompareI"));
		connect(_bICompareI, SIGNAL(clicked()), this, SLOT(OnCompareI()));

		horizont->addSpacing(10);

	//Edges Compare
		_bICompareE = new QPushButton(this);
		horizont->addWidget( _bICompareE );
		_bICompareE->setText("Edges Compare");
		_bICompareE->setIcon(QIcon(":/LODExporter/CompareE"));
		connect(_bICompareE, SIGNAL(clicked()), this, SLOT(OnCompareE()));

		horizont->addSpacing(10);

	// Compare capacity
		_bICompareC = new QPushButton(this);
		horizont->addWidget( _bICompareC );
		_bICompareC->setText("Capacity Compare ");
		_bICompareC->setIcon(QIcon(":/LODExporter/CompareC"));
		connect(_bICompareC, SIGNAL(clicked()), this, SLOT(OnCompareC()));
		
	horizont->addStretch();
	horizont->setSpacing(0);
	///end of tab=======================================





	//start view tab===============================
	horizont = new QHBoxLayout(_toolTabViews);
	_toolTabViews->setLayout(horizont);


	//Solid render
		_bSolid =  new QPushButton(this);		
		horizont->addWidget( _bSolid );
		_bSolid->setText("Solid Render");
		_bSolid->setIcon(QIcon(":/LODExporter/Solid"));
		connect(_bSolid, SIGNAL(clicked()), this, SLOT(OnRenderSolid()));

		horizont->addSpacing(10);

	//Wireframe render
		_bWire =  new QPushButton(this);		
		horizont->addWidget( _bWire );
		_bWire->setText("Wireframe Render");
		_bWire->setIcon(QIcon(":/LODExporter/Wire"));
		connect(_bWire, SIGNAL(clicked()), this, SLOT(OnRenderWireframe()));

		horizont->addSpacing(10);

	//Points render
		_bPoints =  new QPushButton(this);		
		horizont->addWidget( _bPoints );
		_bPoints->setText("Points Render");
		_bPoints->setIcon(QIcon(":/LODExporter/Points"));
		connect(_bPoints, SIGNAL(clicked()), this, SLOT(OnRenderPoints()));

		horizont->addSpacing(10);

	//background color change
		horizont->addWidget( new QLabel( "   Select Background  ", this ) );
		_cSelectBackground = new QComboBox();
		_cSelectBackground->addItem("GREY");
		_cSelectBackground->addItem("GREEN");
		_cSelectBackground->addItem("BLACK");
		connect(_cSelectBackground, SIGNAL(activated(int)), this, SLOT(OnChangeBackground()));
		horizont->addWidget(_cSelectBackground);
		
	horizont->addStretch();
	horizont->setSpacing(0);
	///end of tab=======================================

}


void MyGUIMainWindow::ChangeBackgroundColor(osg::Vec4 in_color)
{
	_background = in_color;
	getView(0)->getCamera()->setClearColor( _background );
	getView(1)->getCamera()->setClearColor( _background );
	getView(2)->getCamera()->setClearColor( _background );
	getView(3)->getCamera()->setClearColor( _background );
}


void MyGUIMainWindow::InitWidgets() 
{
		
	_workCM = new osgGA::TrackballManipulator();
	_w1CM = new osgGA::TrackballManipulator();
	_w2CM = new osgGA::TrackballManipulator();

	txtw1 = new osgText::Text;
	txtw2 = new osgText::Text;
	txtw3 = new osgText::Text;
	txtwp = new osgText::Text;

	txtw1->setText("Working view");
	txtw2->setText("Set distance view");
	txtw3->setText("Comparing view - original model");
	txtwp->setText("Preview");

	_workWidget = addViewWidget( createGraphicsWindow(0,0,100,100), _workCM, txtw1 );
	_viewWidget1 = addViewWidget( createGraphicsWindow(0,0,100,100), _w1CM, txtw2 );
    _viewWidget2 = addViewWidget( createGraphicsWindow(0,0,100,100), _w2CM, txtw3 );
	_previewWidget = addViewWidget( createGraphicsWindow(100,100,1000,650,"Preview",true), new osgGA::TrackballManipulator(), txtwp );

	ChangeBackgroundColor(COLOR_GREY);

	_w1i = new MySnapImage(); 

	getView(1)->getCamera()->setPostDrawCallback(_w1i);


	_mainWidget = new QWidget(this);


    QGridLayout* grid = new QGridLayout();
	QGridLayout* grid2 = new QGridLayout();
    grid2->addWidget( _viewWidget1, 0, 0 );
    grid2->addWidget( _viewWidget2, 1, 0 );
	grid->addWidget( _workWidget, 0, 0);
	grid->addLayout( grid2, 0, 1);

	_mainWidget->setLayout(grid);
	setCentralWidget(_mainWidget);	

	GetViewGroup(1)->setCullCallback(&_psc);
}


void MyGUIMainWindow::InitStatusBar() 
{	
	_bottomBar = new QToolBar( this );
	_bottomBar->setObjectName( "Toolbar bottom" );
	_bottomBar->setMovable( false );
	_bottomBar->setOrientation( Qt::Horizontal );
	addToolBar( Qt::BottomToolBarArea, _bottomBar );

	_lOrigPoly = new QLabel("", this);
	_lSimplyPoly = new QLabel("", this);
	_lPixelSize = new QLabel("", this);
	_lDistance = new QLabel("", this);
	_lTexture = new QLabel("", this);

	_bottomBar->addWidget( new QLabel( "Original: ", this ) );
	_bottomBar->addWidget( _lOrigPoly );


	_bottomBar->addWidget( new QLabel( "   ", this ));
	_bottomBar->addSeparator();
	_bottomBar->addWidget( new QLabel( "   ", this ));


	_bottomBar->addWidget( new QLabel( "Simplified: ", this ) );
	_bottomBar->addWidget( _lSimplyPoly );
	

	_bottomBar->addWidget( new QLabel( "   ", this ));
	_bottomBar->addSeparator();
	_bottomBar->addWidget( new QLabel( "   ", this ));


	_bottomBar->addWidget( new QLabel( "Pixel Size: ", this ) );
	_bottomBar->addWidget( _lPixelSize );
	

	_bottomBar->addWidget( new QLabel( "   ", this ));
	_bottomBar->addSeparator();
	_bottomBar->addWidget( new QLabel( "   ", this ));


	_bottomBar->addWidget( new QLabel( "Distance: ", this ) );
	_bottomBar->addWidget( _lDistance );

	_bottomBar->addWidget( new QLabel( "   ", this ));
	_bottomBar->addSeparator();
	_bottomBar->addWidget( new QLabel( "   ", this ));

	_bottomBar->addWidget( new QLabel( "Selected texture: ", this ) );
	_bottomBar->addWidget( _lTexture );


	_bottomBar->addWidget( new QLabel( "     ", this ));
	_bottomBar->addSeparator();
	_bottomBar->addSeparator();
	_bottomBar->addSeparator();
	_bottomBar->addWidget( new QLabel( "     ", this ));


}



void MyGUIMainWindow::SetRenderSolid() 
{
	osg::PolygonMode* polygonMode = new osg::PolygonMode();
	polygonMode->setMode(osg::PolygonMode::FRONT_AND_BACK, osg::PolygonMode::FILL);
	GetViewGroup(0)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(1)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(2)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(3)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
}


void MyGUIMainWindow::SetRenderWireframe() 
{
	osg::PolygonMode* polygonMode = new osg::PolygonMode();
	polygonMode->setMode(osg::PolygonMode::FRONT_AND_BACK, osg::PolygonMode::LINE);
	GetViewGroup(0)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(1)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(2)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(3)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
}


void MyGUIMainWindow::SetRenderPoints() 
{
	osg::PolygonMode* polygonMode = new osg::PolygonMode();
	polygonMode->setMode(osg::PolygonMode::FRONT_AND_BACK, osg::PolygonMode::POINT);
	GetViewGroup(0)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(1)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(2)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
	GetViewGroup(3)->getOrCreateStateSet()->setAttributeAndModes(polygonMode);
}


QWidget* MyGUIMainWindow::addViewWidget( osgQt::GraphicsWindowQt* gw, osgGA::TrackballManipulator* cm,  osgText::Text *in_text)
{
    osgViewer::View* view = new osgViewer::View;
    addView( view );

    osg::Camera* camera = view->getCamera();
    camera->setGraphicsContext( gw );

    const osg::GraphicsContext::Traits* traits = gw->getTraits();

    camera->setViewport( new osg::Viewport(0, 0, traits->width, traits->height) );
    camera->setProjectionMatrixAsPerspective(30.0f, static_cast<double>(traits->width)/static_cast<double>(traits->height), 1.0f, 10000.0f );

	in_text->setCharacterSize(20);
	in_text->setCharacterSizeMode(osgText::TextBase::SCREEN_COORDS);
	in_text->setDrawMode(osgText::Text::TEXT);
	in_text->setAlignment(osgText::Text::LEFT_TOP);
    in_text->setPosition( osg::Vec3(0, traits->height, 0) );
    in_text->setColor( osg::Vec4(1.0f, 1.0f, 1.0f, 1.0f) );
	in_text->getOrCreateStateSet()->setMode( GL_DEPTH_TEST, osg::StateAttribute::OFF );

	osg::Group *grp = new Group();
	osg::Group *grp_root = new Group();
	osg::Geode *geod = new osg::Geode();
	geod->addDrawable(in_text);

	osg::Projection* pm = new osg::Projection;
	pm->setMatrix(osg::Matrix::ortho2D(0, traits->width, 0, traits->height));
    osg::MatrixTransform* mvm = new osg::MatrixTransform();
    mvm->setMatrix(osg::Matrix::identity());
    mvm->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
    mvm->addChild(pm);
	pm->addChild( geod );
	grp_root->addChild(mvm);
	grp_root->addChild(grp);


    view->setSceneData( grp_root );	
    view->addEventHandler( new osgViewer::StatsHandler );
    view->setCameraManipulator( cm );
	
	//view->getCameraManipulator()->setAutoComputeHomePosition(false);

    return gw->getGLWidget();
}


osgQt::GraphicsWindowQt* MyGUIMainWindow::createGraphicsWindow( int x, int y, int w, int h, const std::string& name, bool windowDecoration )
{
    osg::DisplaySettings* ds = osg::DisplaySettings::instance().get();
    osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
    traits->windowName = name;
    traits->windowDecoration = windowDecoration;
    traits->x = x;
    traits->y = y;
    traits->width = w;
    traits->height = h;
    traits->doubleBuffer = true;
    traits->alpha = ds->getMinimumNumAlphaBits();
    traits->stencil = ds->getMinimumNumStencilBits();
    traits->sampleBuffers = ds->getMultiSamples();
    traits->samples = ds->getNumMultiSamples();

    return new osgQt::GraphicsWindowQt(traits.get());
}





void MyGUIMainWindow::paintEvent( QPaintEvent* event )
{ 
	frame(); 

	MyLOD::CopyCameraManipulator(_w2CM, _w1CM);
	if (MyLODProject::GetProject()->IsSelectedLOD()) {
		MyLOD::CopyCameraManipulator(MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator(), _w1CM);
		MyLODProject::GetProject()->GetSelectedLOD()->SetLODPixels(_psc.PixelSize);
	}
	

	
	//refresh left list SelectLOD
		if (_lSelectLOD->count() != MyLODProject::GetProject()->GetLODCount() + 1 && MyLODProject::GetProject()->GetLODCount() > 0) {
			_lSelectLOD->clear();
			_lSelectLOD->addItem(new QListWidgetItem(""));
			for (int t=0; t<MyLODProject::GetProject()->GetLODCount(); t++) {
				_lSelectLOD->addItem(new QListWidgetItem(""));
			}	
		}
			
		//refresh distances and captions
		if (MyLODProject::GetProject()->GetLODCount() > 0) {
			_lSelectLOD->item(0)->setText(
				"Start-dist: 0\nORIGINAL\nPolygons: " + QString::number(MyLODProject::GetProject()->GetOriginalPolyCount()) + 
				"\nEnd-dist: " + 
				QString::number(MyLODProject::GetProject()->GetLOD(0)->GetLODDistance())
				+ "\n"
			);
		}

		int count = MyLODProject::GetProject()->GetLODCount();
		for (int t=0; t<count; t++) {
			_lSelectLOD->item(t+1)->setText(
				"\nStart-dist: " + QString::number(MyLODProject::GetProject()->GetLOD(t)->GetLODDistance()) + 
				"\nLOD - " + QString::number(count-t-1) + 				
				"\nPixel Size: " + QString::number(MyLODProject::GetProject()->GetLOD(t)->GetLODPixels()) + 
				"\nPolygons: " + QString::number(MyLODProject::GetProject()->GetLOD(t)->GetPolyCount()) +
				"\nEnd-dist: " + 
				(MyLODProject::GetProject()->GetLODCount() == t+1 ? "INF" : QString::number(MyLODProject::GetProject()->GetLOD(t+1)->GetLODDistance())) 
				+ "\n"
			);
		}	
		

		//if (!MyLODProject::GetProject()->IsSelectedOriginal() && (_lSelectLOD->currentRow()-1) != MyLODProject::GetProject()->GetSelectedLODIndex())
		//	_lSelectLOD->setCurrentRow(MyLODProject::GetProject()->GetSelectedLODIndex()+1);
	//=============================		


	//refresh bottom bar
		
		if (MyLODProject::GetProject()->IsSelectedLOD()) {
			_lPixelSize->setText(QString::number(MyLODProject::GetProject()->GetSelectedLOD()->GetLODPixels()));
			_lDistance->setText(QString::number(MyLODProject::GetProject()->GetSelectedLOD()->GetLODDistance()));
			_lSimplyPoly->setText(QString::number(MyLODProject::GetProject()->GetSelectedLOD()->GetPolyCount()));
			//size of textures
			QString txt;
			osg::Image* img;
			std::string name = _cSelectTexture->currentText().toStdString();

			img = MyLODProject::GetProject()->GetOriginalTextureStore()->GetImage(name);
			if (img)
				txt = "(" + QString::number(img->s()) + "x" + QString::number(img->t()) + ")";

			img = MyLODProject::GetProject()->GetSelectedLOD()->GetTextureStore()->GetImage(name);
			if (img)
				txt = QString::number(img->s()) + "x" + QString::number(img->t()) + " " + txt;

			_lTexture->setText(txt);
		}
		else {
			_lPixelSize->setText("-");
			_lDistance->setText("-");
			_lSimplyPoly->setText("-");
			_lTexture->setText("-");
		}
		if (MyLODProject::GetProject()->GetOriginalPolyCount() > -1) 
			_lOrigPoly->setText(QString::number(MyLODProject::GetProject()->GetOriginalPolyCount()));
		else 
			_lOrigPoly->setText("-");


	//======================


	//proces simplifikacie\==============================
	if (_toSimplify > 0)
		_toSimplify--;

	if (_toSimplify == 1) {
		float ratio = ((float)_dsRatio->value()) / 100.0;
		float err = ((float)_dsError->value()) / 100.0;
		int simplifier = _cSelectSimplifier->currentIndex();
		
		//ak je vypnuta max chyba
		if (!_checkError->isChecked())
			err = FLT_MAX;
			
		//odstranenie hran kratsich ako pixel
		if (simplifier == MyLOD::EDGES_SHORTER_PIXEL)
			err = _psc.PixelLength;

		QTime myTimer;
		myTimer.start();		
		MyLODProject::GetProject()->GetSelectedLOD()->Simplify(simplifier, ratio, err, _checkEdges->isChecked());		
		QMessageBox::information(NULL, "Info", "Simplification is completed in " + QString::number(myTimer.elapsed()) + " miliseconds");
		RefreshGUI();		
	}
	else if (_toSimplify == 0) {
		GUIEnable(true);
		_toSimplify = -1;
	}	
	//=====================================================


	
	//proces autoconvertu==================================
	if (_toAutoConvert == 4) {
		GUIEnable(false);
		_menuBar->setEnabled(true);
	}
	else if (_toAutoConvert == 2) {
		MyLODProject::GetProject()->AutoConvert();
		MyLOD::CopyCameraManipulator(_w1CM,  MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator());
		RefreshGUI();	
	}
	else if (_toSimplify == 0) {
		GUIEnable(true);
		_toAutoConvert = -1;
	}
	if (_toAutoConvert > 0)
		_toAutoConvert--;
	//=====================================================



	//compare image	|| edge
	if ((_toImageCompare == 10 || _toImageCompare == 30 || _toImageCompare == 40) && !_w1i->getSnapImageOnNextFrame()) {
		img1 = new osg::Image(*(_w1i->GetImage()));
		_toImageCompare--;
		_last_LOD = MyLODProject::GetProject()->GetSelectedLODIndex();
		GetViewGroup(1)->removeAllChildren();
		GetViewGroup(1)->addChild(MyLODProject::GetProject()->GetOriginalModel());
		MyLODProject::GetProject()->SelectOriginal();
		_w1i->setSnapImageOnNextFrame(true);		
	}
	else if ((_toImageCompare == 9 || _toImageCompare == 29 || _toImageCompare == 39) && !_w1i->getSnapImageOnNextFrame()) {
		_toImageCompare--;
		img2 = new osg::Image(*(_w1i->GetImage()));
		MyLODProject::GetProject()->SelectLOD(_last_LOD);
		RefreshGUI();				
	}
	else if (_toImageCompare == 8 || _toImageCompare == 28) {		
		MyImageCompare *ic = new MyImageCompare();
		if (_toImageCompare == 8)
			ic->CompareImages(img1, img2, _background);
		else if (_toImageCompare == 28)
			ic->CompareSobels(img1, img2, _background);
		_imageCompareWindow->SetData(ic);
		_imageCompareWindow->show();
		_toImageCompare = 0;
	}
	/* pre testovacie ucely
	else if (_toImageCompare == 38) {

			QFile file("looog.txt");
			if (!file.open(QIODevice::Append)) {
				std::cerr << "Cannot open file for writing: "
						  << qPrintable(file.errorString()) << std::endl;
				return;
			}

			QTextStream FILE_STREAM(&file);


		MyImageCompare *ic1 = new MyImageCompare();
		MyImageCompare *ic2 = new MyImageCompare();
		ic1->CompareImages(img1, img2, _background);
		ic2->CompareSobels(img1, img2, _background);
		FILE_STREAM << "    IE: " << ic1->GetBadPixelsError();
		FILE_STREAM.flush();
		FILE_STREAM << "    EE: " << ic2->GetBadPixelsError() << "\n";
		FILE_STREAM.flush();
		_toImageCompare = 0;
	}*/
	
}






void MyGUIMainWindow::RefreshGUI() 
{
	if (!MyLODProject::GetProject()->IsInit() || !MyLODProject::GetProject()->IsLoaded()) {
		_toolBar->setEnabled(false);
		_mainWidget->setEnabled(false);
		return;
	}
	else  {
		_toolBar->setEnabled(true);
		_mainWidget->setEnabled(true);
	}
		



	//refresh Toolbar============================================
		

	//refresh checkbox
	if (_checkError->isChecked())
		_dsError->setEnabled(true);
	else 
		_dsError->setEnabled(false);


	//refresh LOD button "-"
		if (!MyLODProject::GetProject()->IsDeletePossible())
			_bRemoveLOD->setEnabled(false);
		else
			_bRemoveLOD->setEnabled(true);




	//Refresh Undo
		if (MyLODProject::GetProject()->IsSelectedLOD() && !MyLODProject::GetProject()->GetSelectedLOD()->IsUndoPossible())
			_bUndo->setEnabled(false);
		else
			_bUndo->setEnabled(true);

	//Refresh Redo
		if (MyLODProject::GetProject()->IsSelectedLOD() && !MyLODProject::GetProject()->GetSelectedLOD()->IsRedoPossible())
			_bRedo->setEnabled(false);
		else
			_bRedo->setEnabled(true);

	//refresh textures
		_cSelectTexture->clear();
		_cSelectTexture->addItems(MyLODProject::GetProject()->GetTexturesNames());
		if (_cSelectTexture->count() == 0)
			_bResizeTexture->setEnabled(false);
		else 
			_bResizeTexture->setEnabled(true);


	//==========================================================


	//Refresh Widgets===========================================
		if (MyLODProject::GetProject()->IsSelectedLOD()) {				
			GetViewGroup(0)->removeAllChildren();
			GetViewGroup(0)->addChild(MyLODProject::GetProject()->GetSelectedLOD()->GetNode());

			GetViewGroup(1)->removeAllChildren();
			GetViewGroup(1)->addChild(MyLODProject::GetProject()->GetSelectedLOD()->GetNode());
			
			GetViewGroup(2)->removeAllChildren();
			GetViewGroup(2)->addChild(MyLODProject::GetProject()->GetOriginalModel());
		}
		else if (MyLODProject::GetProject()->IsSelectedOriginal()) {
			GetViewGroup(0)->removeAllChildren();
			GetViewGroup(0)->addChild(MyLODProject::GetProject()->GetOriginalModel());			

			GetViewGroup(1)->removeAllChildren();
			GetViewGroup(1)->addChild(MyLODProject::GetProject()->GetOriginalModel());
			getView(1)->setCameraManipulator(getView(1)->getCameraManipulator());
			
			GetViewGroup(2)->removeAllChildren();
			GetViewGroup(2)->addChild(MyLODProject::GetProject()->GetOriginalModel());
			getView(2)->setCameraManipulator(getView(2)->getCameraManipulator());
		}


	//==========================================================

}






void MyGUIMainWindow::OnSelectLOD()
{
	if (_lSelectLOD->currentRow() == 0) {
		MyLODProject::GetProject()->SelectOriginal();
	}
	else {
		if (_lSelectLOD->currentRow() == -1 || (_lSelectLOD->currentRow()-1) == MyLODProject::GetProject()->GetSelectedLODIndex()) 
			return;
		MyLODProject::GetProject()->SelectLOD(_lSelectLOD->currentRow()-1);
		MyLOD::CopyCameraManipulator(_w1CM,  MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator());
	}	
	RefreshGUI();
}


void MyGUIMainWindow::OnAddLOD()
{
	if (MyLODProject::GetProject()->IsSelectedLOD())
		MyLODProject::GetProject()->CreateLOD(MyLODProject::GetProject()->GetSelectedLODIndex());
	else
		MyLODProject::GetProject()->CreateLOD();
	RefreshGUI();
}


void MyGUIMainWindow::OnRemoveLOD()
{
	if (MyLODProject::GetProject()->IsSelectedOriginal())
		return;
	MyLODProject::GetProject()->DeleteSelectedLOD();
	MyLOD::CopyCameraManipulator(_w1CM,  MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator());
	RefreshGUI();
}


void MyGUIMainWindow::OnProjectNew()
{
	QString fileName = QFileDialog::getOpenFileName(NULL, "Select model for LOD", NULL, "*.osgt *.osgb *.osg *.obj *.3ds", 0, 0 );

	if( fileName.isEmpty() )
         return;

	setWindowTitle(fileName);

	_capacity = 0.0;

	if (!MyLODProject::GetProject()->LoadOrigModel(fileName))
		return;

	MyLODProject::GetProject()->SelectLOD(0);

	MyLOD::CopyCameraManipulator(_w1CM, MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator());
	MyLOD::CopyCameraManipulator(_workCM, _w1CM);

	RefreshGUI();
	
	_lSelectLOD->setCurrentRow(1);	
}


void MyGUIMainWindow::OnSimplify()
{
	_toSimplify = 3;
	GUIEnable(false);
	/* pre testovacie ucely
	QFile file("looog.txt");
	if (!file.open(QIODevice::Append)) {
		std::cerr << "Cannot open file for writing: "
				  << qPrintable(file.errorString()) << std::endl;
		return;
	}

	QTextStream FILE_STREAM(&file);




	float ratio = ((float)_dsRatio->value()) / 100.0;
		float err = ((float)_dsError->value()) / 100.0;
		int simplifier = _cSelectSimplifier->currentIndex();
		
		//ak je vypnuta max chyba
		if (!_checkError->isChecked())
			err = FLT_MAX;
			
		//odstranenie hran kratsich ako pixel
		if (simplifier == MyLOD::EDGES_SHORTER_PIXEL)
			err = _psc.PixelLength;

		QTime myTimer;
		myTimer.start();		
		MyLODProject::GetProject()->GetSelectedLOD()->Simplify(simplifier, ratio, err, _checkEdges->isChecked());	
		FILE_STREAM << "S(" << simplifier << ", " << ratio << ") : " << QString::number(myTimer.elapsed());
		

		FILE_STREAM.flush();
	


	if (_capacity == 0.0) {
		//QMessageBox::information(NULL, "Info", "Capacity of original model is not computed. It will be computed now and it may takes several minutes.");
		MyCapacityCounter cc_o;
		_capacity = cc_o.GetCapacity(MyLODProject::GetProject()->GetOriginalModel());
	}
	
	//QMessageBox::information(NULL, "Info", "Program will compute capacity of simplified model. It may takes several minutes.");	
	MyCapacityCounter cc;
	double count = cc.GetCapacity(MyLODProject::GetProject()->GetSelectedLOD()->GetNode());
	
	FILE_STREAM << "     C: " << QString::number(100.0 - (count / _capacity) * 100.0);

	FILE_STREAM.flush();


	_w1i->setSnapImageOnNextFrame(true);
		_toImageCompare = 40;


	QMessageBox::information(NULL, "Info", "End");

		RefreshGUI();		*/
}


void MyGUIMainWindow::OnToolsAutoConvert()
{
	if (QMessageBox::question(this, "Warning", "This action may take several minutes. Do you wish to continue?", QMessageBox::Yes|QMessageBox::No) != QMessageBox::Yes)
		return;	
	_toAutoConvert = 4;	
}


void MyGUIMainWindow::OnCheckError()
{
	RefreshGUI();
}


void MyGUIMainWindow::OnUndo()
{
	if (MyLODProject::GetProject()->GetSelectedLOD()->IsUndoPossible())
		MyLODProject::GetProject()->GetSelectedLOD()->Undo();
	RefreshGUI();
}


void MyGUIMainWindow::OnRedo()
{
	if (MyLODProject::GetProject()->GetSelectedLOD()->IsRedoPossible())
		MyLODProject::GetProject()->GetSelectedLOD()->Redo();
	RefreshGUI();
}



void MyGUIMainWindow::OnToolsPreview()
{
	GetViewGroup(3)->removeAllChildren();
	GetViewGroup(3)->addChild(MyLODProject::GetProject()->GetCompleteLOD(osg::LOD::DISTANCE_FROM_EYE_POINT));
	_previewWidget->show();
}


void MyGUIMainWindow::OnProjectELOD_Distance()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as Distance LOD ...",  QString::null, "*.osgt");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsLOD(filename, osg::LOD::DISTANCE_FROM_EYE_POINT))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}

void MyGUIMainWindow::OnProjectEPLOD_Distance()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as Distance PagedLOD ...",  QString::null, "*.osgt");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsPagedLOD(filename, osg::LOD::DISTANCE_FROM_EYE_POINT))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}

void MyGUIMainWindow::OnProjectELOD_Pixels()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as PixelSize LOD ...",  QString::null, "*.osgt");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsLOD(filename, osg::LOD::PIXEL_SIZE_ON_SCREEN))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}

void MyGUIMainWindow::OnProjectEPLOD_Pixels()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as PixelSize PagedLOD ...",  QString::null, "*.osgt");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsPagedLOD(filename, osg::LOD::PIXEL_SIZE_ON_SCREEN))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}


void MyGUIMainWindow::OnProjectEOBJ()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as OBJ ...",  QString::null, "*.obj");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsOBJ(filename))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}


void MyGUIMainWindow::OnProjectEImpostor()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save as Impostor ...",  QString::null, "*.osgt");
	if( filename.isEmpty() )
		return;
	if (!MyLODProject::GetProject()->ExportAsImpostor(filename))
		QMessageBox::information(NULL, "Error", "Error with saving!");
}

void MyGUIMainWindow::OnRenderSolid()
{
	SetRenderSolid();
	RefreshGUI();
}


void MyGUIMainWindow::OnRenderWireframe()
{
	SetRenderWireframe();
	RefreshGUI();
}



void MyGUIMainWindow::OnRenderPoints()
{	
	SetRenderPoints();
	RefreshGUI();
}


void MyGUIMainWindow::OnChangeBackground()
{
	if (_cSelectBackground->currentIndex() == GREY) {
		ChangeBackgroundColor(COLOR_GREY);
	}
	else if (_cSelectBackground->currentIndex() == GREEN) {
		ChangeBackgroundColor(COLOR_GREEN);
	}
	else if (_cSelectBackground->currentIndex() == BLACK) {
		ChangeBackgroundColor(COLOR_BLACK);
	}

	RefreshGUI();
}


void MyGUIMainWindow::OnTextureResize()
{
	if (_cSelectTexture->count() == 0)
		return;

	std::string name = _cSelectTexture->currentText().toStdString();

	osg::Image* img = MyLODProject::GetProject()->GetOriginalTextureStore()->GetImage(name);
	MyLODProject::GetProject()->GetSelectedLOD()->GetTextureStore()->SetTextureOriginal(name, img);

	if (_cSelectTextureQuality->currentIndex() > 0) {
		MyLODProject::GetProject()->GetSelectedLOD()->GetTextureStore()->ResizeTexture(name, (int)pow(2.0, _cSelectTextureQuality->currentIndex()));
	}

	RefreshGUI();
}

void MyGUIMainWindow::OnRepair()
{
	osg::Node* node;
	if (MyLODProject::GetProject()->IsSelectedLOD()) {
		node = MyLODProject::GetProject()->GetSelectedLOD()->GetNode();
	}
	else if (MyLODProject::GetProject()->IsSelectedOriginal() && MyLODProject::GetProject()->IsLoaded()) {
		node = MyLODProject::GetProject()->GetOriginalModel();
	}
	else 
		return;

	//ak sa detekuju textury - upozornenie
	if (MyLODProject::GetProject()->GetTexturesNames().count() > 0) {
		if (QMessageBox::question(this, "Question", "Textures are detected in model. This procedure can repair vertices but their textures coordinations will be spoiled. This procedure may takes several minutes. Do you want to continue?", QMessageBox::Yes|QMessageBox::No) != QMessageBox::Yes) 
			return;
	}
	else 
		QMessageBox::information(NULL, "Info", "This procedure may takes several minutes.");	
	
	MyVertexRepair vr;
	node->accept(vr);	

	QMessageBox::information(NULL, "Info", "Procedure Repair is completed.");
}

void MyGUIMainWindow::OnProjectSave()
{
	QString filename = QFileDialog::getSaveFileName(this, "Save project as ...",  QString::null, "*.lodproj");
	if( filename.isEmpty() )
		return;
	MyLODProject::GetProject()->SaveProject(filename);
}

void MyGUIMainWindow::OnProjectLoad()
{
	QString fileName = QFileDialog::getOpenFileName(NULL, "Load existing project", NULL, "*.lodproj", 0, 0 );
	if( fileName.isEmpty() )
         return;

	MyLODProject::GetProject()->LoadProject(fileName);
	MyLOD::CopyCameraManipulator(_w1CM, MyLODProject::GetProject()->GetSelectedLOD()->GetCameraManipulator());
	MyLOD::CopyCameraManipulator(_workCM, _w1CM);

	RefreshGUI();
}


void MyGUIMainWindow::OnCompareI()
{
	if(_w1i)
	{
		_w1i->setSnapImageOnNextFrame(true);
		_toImageCompare = 10;
	}
}


void MyGUIMainWindow::OnCompareE()
{
	if(_w1i)
	{
		_w1i->setSnapImageOnNextFrame(true);
		_toImageCompare = 30;
	}
}


void MyGUIMainWindow::OnCompareC()
{

	if (_capacity == 0.0) {
		QMessageBox::information(NULL, "Info", "Capacity of original model is not computed. It will be computed now and it may takes several minutes.");
		MyCapacityCounter cc_o;
		_capacity = cc_o.GetCapacity(MyLODProject::GetProject()->GetOriginalModel());
	}
	
	QMessageBox::information(NULL, "Info", "Program will compute capacity of simplified model. It may takes several minutes.");	
	MyCapacityCounter cc;
	double count = cc.GetCapacity(MyLODProject::GetProject()->GetSelectedLOD()->GetNode());
	
	QMessageBox::information(NULL, "Result", "Capacity of simplified model is different around " + QString::number(100.0 - (count / _capacity) * 100.0) + " %");

}


void MyGUIMainWindow::OnBillboardCreate()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD())
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetActiveBillboard(true);
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardDelete()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard())
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetActiveBillboard(false);	
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardLoadImage()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard())
		return;

	QString fileName = QFileDialog::getOpenFileName(NULL, "Load image for billboard", NULL, "*.jpg *.bmp *.dds *.png *.tga", 0, 0 );
	if( fileName.isEmpty() )
         return;

	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardImage(fileName.toStdString());
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardRotate()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard())
		return;

	MyLODProject::GetProject()->GetSelectedLOD()->RotateBillboardTexCoords();
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardFlip()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard())
		return;

	MyLODProject::GetProject()->GetSelectedLOD()->FlipBillboardTexCoords();
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardScale()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;

	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardScale(_dsWidthBillboard->value(), _dsHeightBillboard->value());
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardResetScale()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->ResetBillboardScale();
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardSetModePoint()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardMode(MyLOD::POINT_ROT_EYE);
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardSetModeX()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardMode(MyLOD::X_AXIS);
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardSetModeY()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardMode(MyLOD::Y_AXIS);
	RefreshGUI();
}


void MyGUIMainWindow::OnBillboardSetModeZ()
{
	if (!MyLODProject::GetProject()->IsSelectedLOD() || !MyLODProject::GetProject()->GetSelectedLOD()->isBillboard() )
		return;
	MyLODProject::GetProject()->GetSelectedLOD()->SetBillboardMode(MyLOD::Z_AXIS);
	RefreshGUI();
}
