//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#ifndef MYGUIMAINWINDOW_H
#define MYGUIMAINWINDOW_H

#include <QTimer>
#include <QTime>
#include <QApplication>
#include <QGridLayout>
#include <QFrame>
#include <QToolBar>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QComboBox>
#include <QFileDialog>
#include <QMessageBox>
#include <QListWidget>
#include <QPushButton>
#include <QCheckBox>
#include <QHBoxLayout>

#include <osgViewer/CompositeViewer>
#include <osgViewer/ViewerEventHandlers>
#include <osgUtil/simplifier>
#include <osgUtil/cullvisitor>
#include <osgGA/TrackballManipulator>
#include <osgDB/ReadFile>
#include <osgQt/GraphicsWindowQt>
#include <osg/polygonmode>
#include <osg/ShapeDrawable>

#include <iostream>
#include <QMainWindow>
#include <QMenuBar>
#include <QHBoxLayout>

#include <math.h> 

#include <osgText/Font>
#include <osgText/Text>


#include "MyLODProject.h"
#include "MyPixelSizeCallback.h"
#include "MySnapImage.h"
#include "MyImageCompare.h"
#include "MyGUIImageCompare.h"
#include "MyCapacityCounter.h"




class MyGUIMainWindow : public QMainWindow, public osgViewer::CompositeViewer
{

	Q_OBJECT

public slots:

	void OnSelectLOD();
	void OnAddLOD();
	void OnRemoveLOD();
	void OnCheckError();
	void OnSimplify();
	void OnRedo();
	void OnUndo();

	void OnRenderSolid();
	void OnRenderWireframe();
	void OnRenderPoints();

	void OnProjectNew();
	void OnProjectSave();
	void OnProjectLoad();
	void OnProjectELOD_Distance();
	void OnProjectEPLOD_Distance();
	void OnProjectELOD_Pixels();
	void OnProjectEPLOD_Pixels();
	void OnProjectEOBJ();
	void OnProjectEImpostor();
	void OnToolsPreview();
	void OnToolsAutoConvert();
	void OnCompareI();
	void OnCompareE();
	void OnCompareC();

	void OnBillboardCreate();
	void OnBillboardDelete();
	void OnBillboardLoadImage();
	void OnBillboardRotate();
	void OnBillboardFlip();
	void OnBillboardScale();
	void OnBillboardResetScale();
	void OnBillboardSetModePoint();
	void OnBillboardSetModeX();
	void OnBillboardSetModeY();
	void OnBillboardSetModeZ();

	void OnChangeBackground();

	void OnTextureResize();

	void OnRepair();

public:

	enum BackgroundColor {
		GREY = 0,
		GREEN,
		BLACK
	};

    MyGUIMainWindow();

	void HideAllWindows();

	void InitGUI();

	void InitMenuActions();

	void InitMenu();

	void InitToolBar();

	void InitWidgets();

	void InitStatusBar();

	void InitLeftBar();

	
	void GUIEnable(bool in_bool);


	void ChangeBackgroundColor(osg::Vec4 in_color);


    QWidget* addViewWidget( osgQt::GraphicsWindowQt* gw, osgGA::TrackballManipulator* cm, osgText::Text *in_text );

    osgQt::GraphicsWindowQt* createGraphicsWindow( int x, int y, int w, int h, const std::string& name="", bool windowDecoration=false );


	void RefreshGUI();

	void SetRenderSolid();
	void SetRenderWireframe();
	void SetRenderPoints();

	osg::Group* GetViewGroup(int in_view);


    virtual void paintEvent( QPaintEvent* event );


private:
	MyGUIImageCompare *_imageCompareWindow;
	MyCapacityCounter *_capacityCounter;

	QAction *_maProjectNew;	
	QAction *_maProjectLoad;	
	QAction *_maProjectSave;	
	QAction *_maProjectELODDist;
	QAction *_maProjectEPLODDist;
	QAction *_maProjectELODPix;
	QAction *_maProjectEPLODPix;
	QAction *_maProjectEOBJ;
	QAction *_maProjectEImpostor;
	QAction *_maProjectExit;
	QAction *_maToolsPreview;
	QAction *_maToolsAuto;
	QAction *_maToolsRepair;
	QAction *_maHelpHelp;
	QAction *_maHelpAbout;

	QMenuBar *_menuBar;
	QMenu* _mProject;
	QMenu *_mExportDist;
	QMenu *_mExportPixel;
	QMenu* _mTools;
	QMenu* _mHelp;



	QToolBar *_toolBar;
	QToolBar *_bottomBar;
	QToolBar *_leftBar;
	QWidget *_mainWidget;

	QTabWidget *_toolTabs;
	QWidget *_toolTabMesh;
	QWidget *_toolTabTexture;
	QWidget *_toolTabBillboard;
	QWidget *_toolTabCompares;
	QWidget *_toolTabViews;

	QWidget *_workWidget;
	QWidget *_viewWidget1;
	QWidget *_viewWidget2;
	QWidget *_previewWidget;
	osgGA::TrackballManipulator* _workCM;
	osgGA::TrackballManipulator* _w1CM;
	osgGA::TrackballManipulator* _w2CM;

	osgText::Text *txtw1, *txtw2, *txtw3, *txtwp;
	osg::Geode *txtgw1, *txtgw2, *txtgw3;

	MySnapImage *_w1i;
	osg::Image *img1, *img2;


	int _last_LOD;

	QListWidget *_lSelectLOD;

	QTimer _guiLoopTimer;



	//TOOLBAR - lod project tools
	QComboBox	*_cSelectLOD;
	QPushButton		*_bAddLOD;
	QPushButton		*_bRemoveLOD;

	//TOOLBAR - Simplify
	QComboBox			*_cSelectSimplifier;
	QDoubleSpinBox		*_dsRatio;
	QCheckBox			*_checkError;
	QCheckBox			*_checkEdges;
	QDoubleSpinBox		*_dsError;
	QPushButton			*_bSimplify;
	QPushButton			*_bUndo;
	QPushButton			*_bRedo;

	QComboBox			*_cSelectTexture;
	QComboBox			*_cSelectTextureQuality;
	QPushButton			*_bResizeTexture;

	QPushButton			*_bCreateBillboard;
	QPushButton			*_bDeleteBillboard;
	QPushButton			*_bLoadBillboard;
	QPushButton			*_bRotateBillboard;
	QPushButton			*_bFlipBillboard;
	QDoubleSpinBox		*_dsWidthBillboard;
	QDoubleSpinBox		*_dsHeightBillboard;
	QPushButton			*_bScaleBillboard;
	QPushButton			*_bScaleResetBillboard;
	QPushButton			*_bModeBillboardPoint;
	QPushButton			*_bModeBillboardX;
	QPushButton			*_bModeBillboardY;
	QPushButton			*_bModeBillboardZ;

	QPushButton			*_bICompareI;
	QPushButton			*_bICompareE;
	QPushButton			*_bICompareC;

	QPushButton			*_bSolid;
	QPushButton			*_bWire;
	QPushButton			*_bPoints;



	QLabel			*_lOrigPoly;
	QLabel			*_lSimplyPoly;
	QLabel			*_lPixelSize;
	QLabel			*_lDistance;
	QLabel			*_lTexture;

	QComboBox		*_cSelectBackground;





	int _toSimplify;
	int _toAutoConvert;
	int _toImageCompare;
	double _capacity;

	MyPixelSizeCallback _psc;


	osg::Vec4		_background;

	osg::Vec4 COLOR_BLACK;
	osg::Vec4 COLOR_GREEN;
	osg::Vec4 COLOR_GREY;

};



#endif