//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyImageCompare.h"

//spocita podla vzorca intenzitu
float MyImageCompare::ComputeIntensity(osg::Vec4 in_col)
{
	return 0.2126f * abs(in_col.r()) + 0.7152 * abs(in_col.g()) + 0.0722 * abs(in_col.b());
}

//spocita podla vzorca intenzitu
float MyImageCompare::ComputeIntensity(QRgb in_col)
{	
	osg::Vec4 vec = osg::Vec4((float)qRed(in_col), (float)qGreen(in_col), (float)qBlue(in_col), 1.0);
	return ComputeIntensity(vec);
}

//prevod OSG image do QT Image
QImage* MyImageCompare::OSGImageToQImage(osg::Image* in_img)
{
	osg::Vec4 col;
	QImage* rtv = new QImage(in_img->s(), in_img->t(), QImage::Format_ARGB32);
	for (int x=0; x<in_img->s(); x++) {
		for (int y=0; y<in_img->t(); y++) {
			col = in_img->getColor(x, y);
			rtv->setPixel(x, in_img->t()-y, qRgb(col.r()*255,  col.g()*255, col.b()*255));
		}
	}
	return rtv;
}

//porovnanie dvoch obrazkov podla pixelov a RSME
void MyImageCompare::CompareImages(osg::Image* in_img1, osg::Image* in_img2, osg::Vec4 in_background)
{
	CompareImages(OSGImageToQImage(in_img1), OSGImageToQImage(in_img2), in_background);	
}

//porovnanie dvoch obrazkov podla pixelov a RSME
void MyImageCompare::CompareImages(QImage* in_img1, QImage* in_img2, osg::Vec4 in_background)
{
	_badPixels = 0;
	_badPixelsError = 0.0;
	_completeError = 0.0;
	_pixelsCount = in_img1->width() * in_img1->height();
	double err = 0.0;

	QImage img2 = in_img2->scaled(in_img1->width(), in_img1->height(), Qt::KeepAspectRatio);

	int intensity, intensityA, intensityB;
	QRgb value, col1q, col2q, dif;
	QRgb backq = qRgb(in_background.r()*255,  in_background.g()*255, in_background.b()*255);
	
	//porovnavame kazdy pixel
	_difImage = new QImage(in_img1->width(), in_img1->height(), QImage::Format_ARGB32);
	for (int x=0; x<in_img1->width(); x++) {
		for (int y=0; y<in_img1->height(); y++) {
			col1q = in_img1->pixel(x, y);
			col2q = img2.pixel(x, y);

			//ak je niektora farba pozadie, a druha farba neni pozadie, tak je chyba 100%
			if ((col1q == backq && col2q != backq) || (col2q == backq && col1q != backq)) {
				intensity = 255;
			}
			//ak su obe farby pozadie, chyba je 0
			else if (col1q == backq && col2q == backq) {
				intensity = 0;
			}			
			else {		
				intensity = (int)abs(ComputeIntensity(col1q) - ComputeIntensity(col2q));
			}
			value = qRgb(intensity, intensity, intensity);			
			_difImage->setPixel(x, y, value);
			if (intensity > 0)
				_badPixels++;
			err += pow((double)(intensity*100)/(double)255, 2);
		}
	}
	//vypocitame hodnoty
	if (_badPixels > 0)
		_badPixelsError = std::sqrt(err / (double)_badPixels);
	_completeError = std::sqrt(err / (double)(_pixelsCount));
}


void MyImageCompare::CompareSobels(osg::Image* in_img1, osg::Image* in_img2, osg::Vec4 in_background)
{
	CompareSobels(OSGImageToQImage(in_img1), OSGImageToQImage(in_img2), in_background);
}

void MyImageCompare::CompareSobels(QImage* in_img1, QImage* in_img2, osg::Vec4 in_background)
{
	CompareImages(ComputeSobel(in_img1), ComputeSobel(in_img2), in_background);	
}

//vypocet sobel hran pre image
QImage* MyImageCompare::ComputeSobel(QImage* in_img)
{
	float g1, g2, g;
	float i[3][3];
	QImage* rtv = new QImage(in_img->width(), in_img->height(), QImage::Format_ARGB32);
	for (int x=1; x<in_img->width()-1; x++) {
		for (int y=1; y<in_img->height()-1; y++) {
			//intensity
			for (int a=0; a<3; a++) {
				for(int b=0; b<3; b++) {
					i[a][b] = ComputeIntensity(in_img->pixel(x+a-1, y+b-1));
				}
			}
			//sobel
			g1 = (-i[0][0] -2.0*i[0][1] -i[0][2]) + (i[2][0] +2.0*i[2][1] +i[2][2]);
			g2 = (i[0][0] +2.0*i[1][0] +i[2][0]) + (-i[0][2] -2.0*i[1][2] -i[2][2]);
			g = sqrt(g1*g1 + g2*g2);
			//zapis do image
			rtv->setPixel(x, y, qRgb((int)g, (int)g, (int)g));
		}
	}
	return rtv;
}


QImage* MyImageCompare::GetDifImage()
{
	return _difImage;
}


long MyImageCompare::GetBadPixels()
{
	return _badPixels;
}


long MyImageCompare::GetPixelsCount()
{
	return _pixelsCount;
}


double MyImageCompare::GetBadPixelsError()
{
	return _badPixelsError;
}


double MyImageCompare::GetCompleteError()
{
	return _completeError;
}