//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#ifndef MYLOD_H
#define MYLOD_H


#include <osgViewer/Viewer>
#include <osgGA/CameraManipulator>
#include <osgGA/TrackballManipulator>
#include <osgUtil/Optimizer>
#include <osgDB/readfile>
#include <osgDB/ImagePager>
#include <osgDB/WriteFile>
#include <osg/Drawable>
#include <osg/group>
#include <osg/ProxyNode>
#include <osg/ref_ptr>
#include <osg/NodeVisitor>
#include <osg/PositionAttitudeTransform>
#include <osg/BlendFunc>

#include <vector>
#include <map>
#include <list>

#include <QStringList>

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

#include <ctime>


#include "MyLODHistory.h"

#include "Simplifier.h"

#include "MyVertCount.h"

#include "MyTextureStore.h"


using namespace std;
using namespace osg;



class MyLOD {
	
public:
	MyLOD(Node* in_node = NULL);
	~MyLOD();

		
	static void CopyCameraManipulator(osgGA::TrackballManipulator* A, osgGA::TrackballManipulator* B);


	bool IsUndoPossible(); 
	bool IsRedoPossible();

	void Undo();
	void Redo();

	

	enum EMethod {
		OSG_ORIGINAL_SIMPLIFIER,
		QUADRIC_ERROR_MATRIC,
		SHORTEST_EDGE,
		SHORTEST_EDGE_NORMAL,
		EDGES_SHORTER_PIXEL,
		STAN_MELAX
	};	

	enum EBillboardType {
		POINT_ROT_EYE,
		X_AXIS,
		Y_AXIS,
		Z_AXIS
	};

	//vracia chybu dalsej hrany po simplifikacii
	float Simplify(int in_method, float in_ratio, float in_maxErr, bool in_edges);

	static QStringList GetMethodStrings();


	
	osgGA::TrackballManipulator* GetCameraManipulator();

	float GetLODDistance();

	float GetLODPixels();
	void SetLODPixels(float in_size);

	long GetPolyCount();

	void ComputePolyCount();


	Node* GetNode();

	MyTextureStore* GetTextureStore();



	void CreateBillboard();
	bool isBillboard();
	void SetActiveBillboard(bool in_val);
	void SetBillboardScale(float in_width, float in_height);
	void ResetBillboardScale();
	void SetBillboardImage(std::string in_file);
	void RotateBillboardTexCoords();
	void FlipBillboardTexCoords();
	void SetBillboardMode(EBillboardType in_mode);


private:

	ref_ptr<osgGA::TrackballManipulator> _cm;
	ref_ptr<Node>			_mesh;
	ref_ptr<Group>				_billboardGrp;
	Billboard*					_billboard;
	StateSet*					_billboardSS;
	PositionAttitudeTransform*	_billboardPos;
	Geometry*					_billboardGeom;
	int							_billboardMode;
	MyTextureStore*				_billboarTexStore;

	long					_polyCount;

	bool					_isBillboard;
	bool					_createdBillboard;

	
	MyTextureStore			*_texStore;

	MyLODHistory			*_history;
	
	float					_pixelSize;


	Geometry* CreatePlane();
	void RebuildBillboardPlaneByAxis(EBillboardType in_mode);
	
};




#endif