//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyLODHistory.h"



MyLODHistory::MyLODHistory() 
{
	_history = new QVector<MyLODHistoryLine*>();
	_historyIndex = -1;
}


MyLODHistory::~MyLODHistory()
{
	for (QVector<MyLODHistoryLine*>::iterator it=_history->begin(); it != _history->end(); ++it) {				
		(*it)->savedNode = NULL;
	}
}


bool MyLODHistory::IsUndoPossible() 
{
	return (_historyIndex > 0);
}


bool MyLODHistory::IsRedoPossible() 
{
	return (_historyIndex < _history->size() - 1);
}

// vytvori jeden zaznam v historii
void MyLODHistory::CreateHistoryLine(string in_desc, Node* in_saveNode) 
{

	while (_history->size() > 0 && _historyIndex < _history->size() - 1) {
		_history->back()->savedNode = NULL;
		_history->pop_back();
	}

	time_t rawtime;
	time(&rawtime);

	MyLODHistoryLine* hl = new MyLODHistoryLine();
	hl->desc = in_desc;
	hl->time = rawtime; 
	hl->savedNode = dynamic_cast<osg::Node*>(in_saveNode->clone(COPY_OPTS));

	_history->push_back(hl);

	_historyIndex = _history->size()-1;
}


// vytiahne predchodzi nod z historie podla 
Node* MyLODHistory::Undo() {
	if (IsUndoPossible()) {
		_historyIndex--;
		MyLODHistoryLine *hl = _history->at(_historyIndex);
		return dynamic_cast<osg::Node*>(hl->savedNode->clone(COPY_OPTS));
	}
	return NULL;
}


// vytoahne nasledujuci nod z historie
Node* MyLODHistory::Redo() {
	if (IsRedoPossible()) {
		_historyIndex++;
		MyLODHistoryLine *hl = _history->at(_historyIndex);
		return dynamic_cast<osg::Node*>(hl->savedNode->clone(COPY_OPTS));
	}
	return NULL;
}

string MyLODHistory::GetListOfHistory() {
	
	string rtv = "";
	char buffer[80];
	struct tm * timeinfo;
	
	for (QVector<MyLODHistoryLine*>::iterator it=_history->begin(); it != _history->end(); ++it) {		
		strftime(buffer, 80, "%I:%M%", localtime(&((*it)->time)));
		rtv += "[ " + string(buffer)  + " ]   " + (*it)->desc + '\n';
	}

	return rtv;
}