//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyLODProject.h"

MyLODProject::MyLODProject()
{
	_lodIndex = -1;
	_polyCount = -1;
	_isInit = false;
	_isLoadedOrig = false;
	_orig_texStore = new MyTextureStore();
}

MyLODProject* MyLODProject::GetProject()
{
	static MyLODProject *_lodProject;

	if (_lodProject == NULL) 
		_lodProject = new MyLODProject();

	return _lodProject;
}




QStringList MyLODProject::GetLODStrings()
{
	QStringList rtv;
	for (int t=0; t<_lods.size(); t++)
		rtv.push_back(QString("LOD - " + QString::number(t)));
	return rtv;
}


QStringList MyLODProject::GetTexturesNames()
{
	QStringList rtv;
	std::vector<std::string> vec = _orig_texStore->GetTexturesNames();
	for (int t=0; t<vec.size(); t++) {
		rtv.push_back(QString::fromStdString(vec.at(t)));
	}
	return rtv;
}


QStringList MyLODProject::GetTexturesQuality()
{
	QStringList rtv;
	rtv.push_back("Original");
	for (int t=2; t<=16; t*=2) {
		rtv.push_back("Original / " + QString::number(t));
	}
	return rtv;
}


void MyLODProject::CreateLOD(int in_fromLOD)
{
	if (!_isInit || !_isLoadedOrig)
		return;

	osg::Node* node = _origRoot;
	if (in_fromLOD > -1)
		node = GetLOD(in_fromLOD)->GetNode();

	MyLOD* new_lod = new MyLOD(node);
	new_lod->GetCameraManipulator()->setCenter(new_lod->GetNode()->getBound().center());
	new_lod->GetCameraManipulator()->setDistance(new_lod->GetNode()->getBound().radius() * 2.5);

	if (in_fromLOD == -1) {
		_lods.push_front(new_lod);
		_lodIndex = 0;
	}
	else {
		MyLOD::CopyCameraManipulator(new_lod->GetCameraManipulator(), GetLOD(in_fromLOD)->GetCameraManipulator());
		_lods.insert(in_fromLOD+1, new_lod);	
		_lodIndex++;
	}
}


bool MyLODProject::IsDeletePossible()
{
	return (_lods.size() > 1);
}


bool MyLODProject::IsInit()
{
	return _isInit;
}


bool MyLODProject::IsLoaded()
{
	return _isLoadedOrig;
}
 

void MyLODProject::DeleteSelectedLOD()
{

	MyLOD* to_kill = GetSelectedLOD();
	if (to_kill != NULL) {
		_lods.remove(_lodIndex);
		delete to_kill;
		_lodIndex = 0;
	}
}


void MyLODProject::SelectLOD(int in_index)
{
	_lodIndex = in_index;
	_selectedOriginal = false;
}


bool MyLODProject::IsSelectedLOD() 
{
	return (!_selectedOriginal && _lodIndex < _lods.size() && _lodIndex >= 0);
}


void MyLODProject::SelectOriginal()
{
	_selectedOriginal = true;
	_lodIndex = -1;
}


bool MyLODProject::IsSelectedOriginal()
{
	return _selectedOriginal;
}


int MyLODProject::GetSelectedLODIndex()
{
	return _lodIndex;
}


int MyLODProject::GetLODCount()
{
	return _lods.size();
}


MyLOD* MyLODProject::GetSelectedLOD()
{
	return GetLOD(_lodIndex);
}

MyLOD* MyLODProject::GetLOD(int in_index) 
{
	if (in_index < _lods.size() && in_index >= 0)
		return _lods[in_index];
	return NULL;
}


bool MyLODProject::LoadOrigModel(QString in_path)
{
	_origRoot = NULL;
	_origRoot = osgDB::readNodeFile(in_path.toStdString());
	_orig_texStore->Clear();

	if (_origRoot == NULL) {
		return false;
	}

	osgUtil::Optimizer* my_opt = new osgUtil::Optimizer();
	my_opt->optimize(_origRoot, osgUtil::Optimizer::ALL_OPTIMIZATIONS - osgUtil::Optimizer::TEXTURE_ATLAS_BUILDER); 		

	osgUtil::SmoothingVisitor sv;
	_origRoot->accept(sv);

	_origRoot->accept(*_orig_texStore);

	MyVertCount pc;
	_origRoot->accept(pc);
	_polyCount = pc.GetPolyCount();
	_isLoadedOrig = true;
	Init();	
	CreateLOD();
	return true;
}


Node* MyLODProject::GetOriginalModel()
{
	return _origRoot;
}


long MyLODProject::GetOriginalPolyCount()
{
	return _polyCount;
}


void MyLODProject::ClearLODs()
{
	_lodIndex = 0;
	while (_lods.size() > 0)
		DeleteSelectedLOD();
}


void MyLODProject::Init()
{	
	_isInit = true;
	ClearLODs();				
}


void MyLODProject::ComputeImagesNames(bool in_folderPerFile, bool in_completeNames)
{
	GetOriginalTextureStore()->ComputeImagesNames(in_folderPerFile, in_completeNames);
	for (int t=0; t<_lods.size(); t++) 
		_lods[t]->GetTextureStore()->ComputeImagesNames(in_folderPerFile, in_completeNames);
}


void MyLODProject::RestoreImagesNames()
{
	GetOriginalTextureStore()->RestoreImagesNames();
	for (int t=0; t<_lods.size(); t++)
		_lods[t]->GetTextureStore()->RestoreImagesNames();
}


void MyLODProject::SetImagesPath(QString in_dir)
{
	GetOriginalTextureStore()->SetImagePath(in_dir.toStdString());
	for (int t=0; t<_lods.size(); t++)
		_lods[t]->GetTextureStore()->SetImagePath(in_dir.toStdString());
}


void MyLODProject::SaveImages(QString in_path, bool in_folderPerFile, bool in_completeNames)
{	
	if (!QDir(in_path).exists())
		QDir().mkdir(in_path);

	//save images
	GetOriginalTextureStore()->SaveImages(in_path, in_folderPerFile, in_completeNames);
	for (int t=0; t<_lods.size(); t++)
		_lods[t]->GetTextureStore()->SaveImages(in_path, in_folderPerFile, in_completeNames);
}

LOD* MyLODProject::GetCompleteLOD(PagedLOD::RangeMode in_mode)
{

	QVector<float> ranges;
	ranges.push_back(0.0f);
	for (int t=0; t<_lods.size(); t++) {
		if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN) {
			ranges.push_back(_lods[t]->GetLODPixels());
		}
		else {
			ranges.push_back(_lods[t]->GetLODDistance());
		}
		
	}
	ranges.push_back(FLT_MAX);
	std::sort(ranges.begin(), ranges.end());

	if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN) 
		std::reverse(ranges.begin(), ranges.end());
	
	LOD* rtv = new LOD();
	rtv->setRangeMode(in_mode);

	rtv->addChild(_origRoot, 0, 0);	
	for (int t=1; t<=_lods.size(); t++)
		rtv->addChild(_lods[t-1]->GetNode(), 0, 0);

	for (int t=0; t<ranges.size()-1; t++) {
		if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN)
			rtv->setRange(t, ranges[t+1], ranges[t]);
		else 
			rtv->setRange(t, ranges[t], ranges[t+1]);
	}

	return rtv;
}


bool MyLODProject::ExportAsLOD(QString in_file, PagedLOD::RangeMode in_mode)
{
	QFileInfo fi(in_file);
	
	//create image dir
	QString img_folder_rel = fi.baseName() + "_images";
	QString img_path = fi.path() + "/" + img_folder_rel;
	SaveImages(img_path);
	//prepare tree to save
	SetImagesPath(img_folder_rel);
	ComputeImagesNames();
	if (!osgDB::writeNodeFile(*(GetProject()->GetCompleteLOD(in_mode)), in_file.toStdString()))
		return false;
	RestoreImagesNames();
	return true;
}


bool MyLODProject::ExportAsImpostor(QString in_file)
{
	QFileInfo fi(in_file);
	
	//create image dir
	QString img_folder_rel = fi.baseName() + "_images";
	QString img_path = fi.path() + "/" + img_folder_rel;
	SaveImages(img_path);
	//prepare tree to save
	SetImagesPath(img_folder_rel);
	ComputeImagesNames();
	
	osg::LOD *lod = GetProject()->GetCompleteLOD(PagedLOD::DISTANCE_FROM_EYE_POINT);
	osgSim::Impostor* impos = new osgSim::Impostor();
	impos->setRangeList(lod->getRangeList());
	for (int t=0; t<lod->getNumChildren(); t++)
		impos->addChild(lod->getChild(t));

	float last_dist = lod->getRangeList().at(lod->getRangeList().size()-1).first;
	impos->setImpostorThreshold(last_dist);
	if (!osgDB::writeNodeFile(*(impos), in_file.toStdString()))
		return false;
	
	RestoreImagesNames();
	return true;
}


bool MyLODProject::ExportAsOBJ(QString in_file)
{
	QFileInfo fi(in_file);
	
	//txt file
	QString txt = "";
	//create image dir
	QString img_folder_rel = fi.baseName() + "_images";
	QString img_path = fi.path() + "/" + img_folder_rel;
	SaveImages(img_path);
	//prepare tree to save
	SetImagesPath(img_folder_rel);
	ComputeImagesNames();

	osg::LOD *lod = GetProject()->GetCompleteLOD(PagedLOD::DISTANCE_FROM_EYE_POINT);

	bool tmp_b = true;
	QString tmp_file, tmp_file_name;
	for (int t=0; t<lod->getNumChildren(); t++) {
		tmp_file_name = "/" + fi.baseName() + "_" + QString::number(lod->getNumChildren() - t - 1) + ".obj";
		tmp_file = fi.path() + tmp_file_name;
		txt += QString::number(lod->getRangeList().at(t).first) + " - " + QString::number(lod->getRangeList().at(t).second) + "     " + tmp_file_name + "\n";
		if (!osgDB::writeNodeFile(*(lod->getChild(t)), tmp_file.toStdString()))		
			tmp_b = false;
	}

	QFile txtfile (fi.path() + "/" + fi.baseName() + "_info.txt");
	if ( txtfile.open(QIODevice::ReadWrite) )
	{
		QTextStream outstream(&txtfile);
		outstream << txt;
	}

	if (!tmp_b) {
		QFile::remove(in_file) ;
		for (int t=0; t<_lods.size(); t++) 
			QFile::remove(fi.path() + "/" + fi.baseName() + "_" + QString::number(t) + ".obj") ;
		return false;
	}

	RestoreImagesNames();

	return true;
}


bool MyLODProject::ExportAsPagedLOD(QString in_file, PagedLOD::RangeMode in_mode)
{
	QFileInfo fi(in_file);
	
	//create image dir
	QString img_folder_rel = fi.baseName() + "_images";
	QString img_path = fi.path() + "/" + img_folder_rel;
	SaveImages(img_path);
	//prepare tree to save
	SetImagesPath(img_folder_rel);
	

	//ulozit lod subory
	std::vector<std::string> names;

	ComputeImagesNames();

	QString tmp_file = fi.path() + "/" + fi.baseName() + "_" + QString::number(_lods.size()) + ".osgt";
	if (!osgDB::writeNodeFile(*(_origRoot), tmp_file.toStdString()))
		return false;
	names.push_back(QFileInfo(tmp_file).fileName().toStdString());

	
	int lod_num;
	for (int t=0; t<_lods.size()-1; t++) {
		lod_num = _lods.size()-t-1;
		tmp_file = fi.path() + "/" + fi.baseName() + "_" + QString::number(lod_num) + ".osgt";
		osgDB::writeNodeFile(*(_lods[t]->GetNode()), tmp_file.toStdString());
		names.push_back(QFileInfo(tmp_file).fileName().toStdString());
	}

	//vypocet lod uzlu
		
	PagedLOD* plod = new PagedLOD();	
	plod->setRangeMode(in_mode);

	//dat najnizsi detail do plodu nativne
	if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN) {
		plod->addChild(_lods[_lods.size()-1]->GetNode(), _lods[_lods.size()-1]->GetLODPixels(), FLT_MAX);
	}
	else {
		plod->addChild(_lods[_lods.size()-1]->GetNode(), _lods[_lods.size()-1]->GetLODDistance(), FLT_MAX);
	}
	
	
	//nastavenie mien
	for (int t=0; t<names.size(); t++) {
		plod->setFileName(t+1, names[names.size()-t-1]);
	}

	//nastavenie rozsahov
	QVector<float> ranges;
	ranges.push_back(0.0f);
	for (int t=0; t<_lods.size(); t++) {
		if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN) {
			ranges.push_back(_lods[t]->GetLODPixels());
		}
		else {
			ranges.push_back(_lods[t]->GetLODDistance());
		}
	}
	ranges.push_back(FLT_MAX);
	std::sort(ranges.begin(), ranges.end());

	if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN) 
		std::reverse(ranges.begin(), ranges.end());

	for (int t=0; t<ranges.size()-1; t++) {
		if (in_mode == PagedLOD::PIXEL_SIZE_ON_SCREEN)
			plod->setRange(ranges.size()-t-2, ranges[t+1], ranges[t]);
		else 
			plod->setRange(ranges.size()-t-2, ranges[t], ranges[t+1]);
	}
	
	//zapis PLOD uzlu
	osgDB::writeNodeFile(*plod, in_file.toStdString());
	

	RestoreImagesNames();

	return true;
}


MyTextureStore* MyLODProject::GetOriginalTextureStore()
{
	return _orig_texStore;
}


void MyLODProject::AutoConvert()
{
	//koeficient pre postupny vypocet vzdialenosti a lodov
	int lods = 3;
	float dist_k1 = 1.5;	
	float lod_base = 0.7;	
	float dist_k2 = 2.5;
	
	//vypocet prvotnej vzdialenosti
	float first_dist = dist_k1 * MyLODProject::GetProject()->GetOriginalModel()->getBound().radius() / COS_45;
	float err;
	
	//vytvorenie lodov, ich simplifikacia a nastavenie vzdialenosti	
	ClearLODs();
	for (int t=0; t<lods; t++) {
		CreateLOD();
		err = _lods[0]->Simplify(MyLOD::QUADRIC_ERROR_MATRIC, pow(lod_base, 2*(lods-t)), FLT_MAX, true);
		_lods[0]->GetCameraManipulator()->setDistance(first_dist * pow(dist_k2, (lods-t)) * (1+err));
	}

}


bool MyLODProject::SaveProject(QString in_file)
{
	QFileInfo fi(in_file);
	
	//create osgt file
	QString osgt_file = fi.path() + "/" + fi.baseName() + ".osgt";	

	SaveImages(fi.path() + "/images", true, false);
	//prepare tree to save
	SetImagesPath("/images");
	ComputeImagesNames(true, false);
	if (!osgDB::writeNodeFile(*(GetProject()->GetCompleteLOD(osg::LOD::DISTANCE_FROM_EYE_POINT)), osgt_file.toStdString()))
		return false;
	RestoreImagesNames();

	QFile::rename(osgt_file, in_file);

	return true;
}


bool MyLODProject::LoadProject(QString in_file)
{
	QFileInfo fi(in_file);
	osgUtil::Optimizer* my_opt = new osgUtil::Optimizer();
	osgUtil::SmoothingVisitor sv;
	
	//create osgt file name
	QString osgt_file = fi.path() + "/" + fi.baseName() + ".osgt";
	//rename, load, rename
	QFile::rename(in_file, osgt_file);
	osg::LOD* file_node = (osg::LOD*)osgDB::readNodeFile(osgt_file.toStdString());
	QFile::rename(osgt_file, in_file);

	//init
	Init();
	_orig_texStore->Clear();
	
	//test na children
	if (file_node->getNumChildren() < 2)
		return false;

	//create orig
	_origRoot = NULL;
	_origRoot = dynamic_cast<Node*>(file_node->getChild(0)->clone(osg::CopyOp::DEEP_COPY_ALL));

	my_opt->optimize(_origRoot, osgUtil::Optimizer::ALL_OPTIMIZATIONS - osgUtil::Optimizer::TEXTURE_ATLAS_BUILDER);
	_origRoot->accept(sv);
	_origRoot->accept(*_orig_texStore);

	MyVertCount pc;
	_origRoot->accept(pc);
	_polyCount = pc.GetPolyCount();
	_isLoadedOrig = true;

	//create lods
	MyLOD* new_lod;	
	for (unsigned int t=1; t<file_node->getNumChildren(); t++) {
		my_opt->optimize(file_node->getChild(t), osgUtil::Optimizer::ALL_OPTIMIZATIONS - osgUtil::Optimizer::TEXTURE_ATLAS_BUILDER);
		file_node->getChild(t)->accept(sv);
		new_lod = new MyLOD(file_node->getChild(t));
		new_lod->GetCameraManipulator()->setCenter(new_lod->GetNode()->getBound().center());
		new_lod->GetCameraManipulator()->setDistance(file_node->getRangeList().at(t).first);
		_lods.push_back(new_lod);
	}

	SelectLOD(0);
	
	return true;
}
