//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#ifndef MYLODPROJECT_H
#define MYLODPROJECT_H


#include <osgViewer/Viewer>
#include <osgUtil/Simplifier>
#include <osgUtil/Optimizer>
#include <osgDB/readfile>
#include <osgDB/ImagePager>
#include <osgDB/WriteFile>
#include <osg/Drawable>
#include <osg/group>
#include <osg/ProxyNode>
#include <osg/ref_ptr>
#include <osg/NodeVisitor>
#include <osg/PagedLOD>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/TriStripVisitor>
#include <osgSim/Impostor>

#include <QStringList>
#include <QVector>
#include <QFileInfo>
#include <QTextStream>
#include <QDir>

#include <vector>
#include <map>
#include <list>

#include <iostream>
#include <fstream>
#include <string>

#include <ctime>


#include "MyLOD.h"
#include "MyTextureStore.h"
#include "MyVertexRepair.h"


using namespace std;


#define COS_45 0.70710678118654752440084436210485


class MyLODProject {
	
public:
	
	static MyLODProject* GetProject();

	bool LoadOrigModel(QString in_path);
	Node* GetOriginalModel();
	bool IsLoaded();
	bool IsInit();


	void CreateLOD(int in_fromLOD = -1);

	void AutoConvert();

	void DeleteSelectedLOD();
	bool IsDeletePossible();

	void SelectLOD(int in_index);
	bool IsSelectedLOD();

	void SelectOriginal();
	bool IsSelectedOriginal();
	long GetOriginalPolyCount();

	MyTextureStore* GetOriginalTextureStore();

	int GetSelectedLODIndex();
	MyLOD* GetSelectedLOD();
	int GetLODCount();
	MyLOD* GetLOD(int in_index);

	QStringList GetLODStrings();
	
	QStringList GetTexturesNames();
	QStringList GetTexturesQuality();

	
	LOD* GetCompleteLOD(PagedLOD::RangeMode in_mode);
	bool ExportAsLOD(QString in_file, PagedLOD::RangeMode in_mode);
	bool ExportAsPagedLOD(QString in_file, PagedLOD::RangeMode in_mode);
	bool ExportAsOBJ(QString in_file);
	bool ExportAsImpostor(QString in_file);

	bool SaveProject(QString in_file);
	bool LoadProject(QString in_file);
	

private:
	//singleton
	MyLODProject();
	MyLODProject(MyLODProject const&);
	void operator=(MyLODProject const&);
	
	//init project po nacitani originalneho file
	bool _isInit;
	bool _isLoadedOrig;
	bool _selectedOriginal;
	long _polyCount;

	void ClearLODs();
	void Init();

	void ComputeImagesNames(bool in_folderPerFile = false, bool in_completeNames = true);
	void RestoreImagesNames();
	void SetImagesPath(QString in_dir);
	void SaveImages(QString in_path, bool in_folderPerFile = false, bool in_completeNames = true);

	//LODs
	QVector<MyLOD*> _lods;


	//aktivne vybrany lod
	int _lodIndex;


	osg::Camera *_camera;
	ref_ptr<osg::Node> _origRoot;

	MyTextureStore *_orig_texStore;

	

};




#endif
