//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyTextureStore.h"


MyTextureStore::MyTextureStore() : NodeVisitor( NodeVisitor::TRAVERSE_ALL_CHILDREN )
{
	textSlots = new std::map<std::string, std::vector<MyTextureStoreSlot*>*>();
}


void MyTextureStore::Clear()
{
	textSlots->clear();
}


std::string MyTextureStore::GetFileName(std::string in_path)
{
	std::string new_name = in_path;
	if (new_name.find_last_of("/") != std::string::npos)
		new_name = new_name.substr(new_name.find_last_of("/") + 1, new_name.size() - new_name.find_last_of("/") - 1);
	else
		new_name = new_name.substr(new_name.find_last_of("\\") + 1, new_name.size() - new_name.find_last_of("\\") - 1);
	return new_name;
}


std::string MyTextureStore::GetFilePath(std::string in_file)
{
	std::string path = in_file;
	if (path.find_last_of("/") != std::string::npos)
		path = path.substr(0, path.find_last_of("/") + 1);
	else
		path = path.substr(0, path.find_last_of("\\") + 1);
	return path;
}


// pri priechode spracuje state set a zobere zneho textury 
void MyTextureStore::DoStateSet(osg::StateSet* in_state)
{
	if (in_state == NULL)
		return;

	if (in_state->getNumTextureAttributeLists() == 0)
		return;

	//in_state->setDataVariance(osg::Object::STATIC);

	MyTextureStoreSlot *tss;
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;

	for (int t=0; t<in_state->getTextureAttributeList().size(); t++) {
		tss = new MyTextureStoreSlot();
		tss->Texture = in_state->getTextureAttribute(t, osg::StateAttribute::TEXTURE)->asTexture();	

		tss->OrigFullFileName = tss->Texture->getImage(0)->getFileName();
		tss->OrigFileName = GetFileName(tss->Texture->getImage(0)->getFileName());
		tss->Path = GetFilePath(tss->Texture->getImage(0)->getFileName());
		tss->RestoreImageName();
		tss->UpdateSizes();

		it = textSlots->find(tss->OrigFileName);
		if (it != textSlots->end()) {
			it->second->push_back(tss);
		}
		else {
			std::vector<MyTextureStoreSlot*> *vec = new std::vector<MyTextureStoreSlot*>();
			vec->push_back(tss);
			textSlots->insert(std::pair<std::string, std::vector<MyTextureStoreSlot*>*>(tss->OrigFileName, vec));
		}
		
	}
}


// vypocita mena pre obrazky podla ich velkosti 
void MyTextureStore::ComputeImagesNames(bool in_folderPerFile, bool in_completeNames)
{
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	for (it=textSlots->begin(); it!=textSlots->end(); ++it) {
		for (int i=0; i<it->second->size(); i++) {
			it->second->at(i)->ComputeImageName(in_folderPerFile, in_completeNames);
		}			
	}
}

//nastavi men=a podla originalu
void MyTextureStore::RestoreImagesNames()
{
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	for (it=textSlots->begin(); it!=textSlots->end(); ++it) {
		for (int i=0; i<it->second->size(); i++) {
			it->second->at(i)->RestoreImageName();
		}			
	}
}

//nastavi si cestu kde obrazky budu ... treba strom prisposobit tomu ze obrazky mozno budu v podzlozke
void MyTextureStore::SetImagePath(std::string in_path)
{
	if (in_path == "")
		return;

	std::string new_name, separator = "";
	if (in_path[in_path.size()-1] != '/' && in_path[in_path.size()-1] != '\\') {
		separator = "/";
		if (in_path.find_last_of("\\") != std::string::npos)
			separator = "\\";
	}

	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	for (it=textSlots->begin(); it!=textSlots->end(); ++it) {
		for (int i=0; i<it->second->size(); i++) {
			it->second->at(i)->Path = in_path + separator;
		}			
	}
}

// zobere vsetky textury a ulozi ich na disk podla nastavenej cesty 
void MyTextureStore::SaveImages(QString in_path, bool in_folderPerFile, bool in_completeNames)
{
	std::string in_path_str = in_path.toStdString();
	std::string new_name, separator = "";
	if (in_path_str[in_path_str.size()-1] != '/' && in_path_str[in_path_str.size()-1] != '\\') {
		separator = "/";
		if (in_path_str.find_last_of("\\") != std::string::npos)
			separator = "\\";
	}
	in_path = in_path + QString(separator.c_str());

	osg::Image* img;
	QString file_name, file_folder;
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	for (it=textSlots->begin(); it!=textSlots->end(); ++it) {
		for (int i=0; i<it->second->size(); i++) {
			img = it->second->at(i)->Texture->getImage(0);			
			
			if (in_completeNames)
				file_name = QString(it->second->at(i)->GetCompleteFileName().c_str());
			else 
				file_name = QString(it->second->at(i)->OrigFileName.c_str());

			if (in_folderPerFile) {
				it->second->at(i)->UpdateSizes();
				file_folder = QString(it->second->at(i)->Sizes.c_str()) + "/";
			}
			else 
				file_folder = "";

			if (!QDir(in_path + file_folder).exists())
				QDir().mkdir(in_path + file_folder);

			osgDB::writeImageFile(*(img), (in_path + file_folder + file_name).toStdString());
		}			
	}

}


void MyTextureStore::apply(osg::Geode& geode)
{
	DoStateSet(geode.getStateSet());

	for(unsigned int i=0;i<geode.getNumDrawables();++i)
    {
        if (geode.getDrawable(i))
			DoStateSet(geode.getDrawable(i)->getStateSet());
    }
}


void MyTextureStore::apply(osg::Node& node)
{
	DoStateSet(node.getStateSet());
	traverse(node);
}


std::vector<std::string> MyTextureStore::GetTexturesNames()
{
	std::vector<std::string> ret;
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;

	for (it=textSlots->begin(); it!=textSlots->end(); ++it) {
		ret.push_back(it->first);
	}

	return ret;
}

// nakopiruje image do vsetkych textur ... sluzi pre mozne znovu nastavenie originalu po zmenseni
void MyTextureStore::SetTextureOriginal(std::string in_name, osg::Image* in_origImage)
{
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
		
	it = textSlots->find(in_name);
	if (it == textSlots->end()) 
		return;

	osg::Image* img = dynamic_cast<osg::Image*>(in_origImage->clone(osg::CopyOp::DEEP_COPY_ALL));

	for (int t=0; t<it->second->size(); t++) {
		it->second->at(t)->Texture->setImage(0, img);		
	}

}


void MyTextureStore::ResizeTexture(std::string in_name, int in_rate)
{
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	osg::Image* img;
	
	it = textSlots->find(in_name);
	if (it == textSlots->end()) 
		return;

	int w = -1;
	int h = -1;
	for (int t=0; t<it->second->size(); t++) {
		img = it->second->at(t)->Texture->getImage(0);
		if (w == -1 || h == -1) {
			w = img->s() / in_rate;
			h = img->t() / in_rate;
		}
		img->scaleImage(w, h, img->r());
	}
}


//najde image podla mena
osg::Image* MyTextureStore::GetImage(std::string in_name)
{
	std::map<std::string, std::vector<MyTextureStoreSlot*>*>::iterator it;
	osg::Image* img;
	
	it = textSlots->find(in_name);
	if (it == textSlots->end()) 
		return NULL;
	if (!it->second || !it->second->at(0)->Texture || it->second->at(0)->Texture->getNumImages() == 0)
		return NULL;

	return it->second->at(0)->Texture->getImage(0);
}