//======================================
//=== Copyright (c) 2014 Dušan Hupka ===
//=== BSD License                    ===
//=== xhupka00@stud.fit.vutbr.cz     ===
//======================================



#include "MyVertexRepair.h"


osg::PrimitiveSet* MyVertexRepair::ReplaceIndices(osg::PrimitiveSet* in_ps, std::map<unsigned int, long>* in_ver)
{
	long vert;

	if (in_ps->getType() == osg::PrimitiveSet::DrawArraysPrimitiveType) {
		osg::DrawElementsUShort* ps = new osg::DrawElementsUShort(in_ps->getMode(), 0);
		for (unsigned int t=((osg::DrawArrays*)in_ps)->getFirst(); t<in_ps->getNumIndices(); t++) {
			vert = in_ver->find(t)->second;
			if (vert != t && vert != -1)
				ps->push_back(vert);
			else
				ps->push_back(t);
		}
		return ps;
	}
	else if (in_ps->getType() == osg::PrimitiveSet::DrawElementsUShortPrimitiveType) {
		osg::DrawElementsUShort* ps = new osg::DrawElementsUShort(in_ps->getMode(), 0);
		for (unsigned int t=0; t<in_ps->getNumIndices(); t++) {
			vert = in_ver->find(in_ps->index(t))->second;
			if (vert != in_ps->index(t) && vert != -1)
				ps->push_back(vert);
			else
				ps->push_back(in_ps->index(t));
		}	
		return ps;
	}
	else 
		return in_ps;


}

bool MyVertexRepair::IsNear(osg::Vec3 in_a, osg::Vec3 in_b, float in_near)
{
	return ((in_a-in_b).length() < in_near);
}

void MyVertexRepair::DoDrawable(osg::Drawable* in_draw)
{
	std::vector<osg::PrimitiveSet*> ps_vec;
	osg::Vec3Array* arr = (osg::Vec3Array*)in_draw->asGeometry()->getVertexArray();
	float max_tol = in_draw->getBound().radius()*0.00001;
	float max_tol2 = max_tol*2.0;

	unsigned int m_sv_pos;
	std::map<unsigned int, std::vector<unsigned int>*> m_sv;
	std::map<unsigned int, std::vector<unsigned int>*>::iterator m_sv_it, m_sv_it2;
	for (unsigned int t=0; t<arr->getNumElements(); t++) {
		m_sv_pos = (unsigned int)(arr->at(t).length() / max_tol2);
		m_sv_it = m_sv.find(m_sv_pos);
		if (m_sv_it != m_sv.end()) {
			m_sv_it->second->push_back(t);
		}
		else {
			std::vector<unsigned int>* new_vec = new std::vector<unsigned int>();
			new_vec->push_back(t);
			m_sv.insert(std::pair<unsigned int, std::vector<unsigned int>*>(m_sv_pos, new_vec));
		}
	}
		

	//nastavenie spracovanie(prepisovanie) vertexu - zatial vsetky na -1
	std::map<unsigned int, long> m_ver;
	for (unsigned int t=0; t<arr->getNumElements(); t++) 
		m_ver.insert(std::pair<unsigned int, long>(t, -1));


	unsigned int a, b;
	std::vector<unsigned int> tmp_vec;
	for (m_sv_it=m_sv.begin(); m_sv_it!=m_sv.end(); ++m_sv_it) {
			
		//spravime pomocny vektor s bodmi
		tmp_vec.clear();
		tmp_vec.insert(tmp_vec.end(), m_sv_it->second->begin(), m_sv_it->second->end());

		//pridame k bodom aj druhy sektor (koli hranicnym bodom)
		m_sv_it2 = m_sv.find(m_sv_it->first+1);
		if (m_sv_it2 != m_sv.end())
			tmp_vec.insert(tmp_vec.end(), m_sv_it2->second->begin(), m_sv_it2->second->end());

		//ak v sektore niesu ani 2 vertexy, ideme dalej
		if (tmp_vec.size() < 2)
			continue;

		//prechadzame vsetky pary vertexov v danych dvoch sektoroch
		for (int i=0; i<tmp_vec.size(); i++) {
				
			//ak uz vertex je spracovany, preskocime ho
			a = tmp_vec.at(i);
			if (m_ver.find(a)->second > -1)
				continue;

			//skusime vertex pospajat s ostatnymi v kategorii
			for (int t=i+1; t<tmp_vec.size(); t++) {
					
				//ak uz je priradeny, continue
				b = tmp_vec.at(t);
				if (m_ver.find(b)->second > -1)
					continue;

				//zistenie ci sa maju spojit
				if (!IsNear(arr->at(a), arr->at(b), max_tol))
					continue;

				//vertexy boli spracovane
				m_ver.find(a)->second = a; // indikator ze sa nema prepisovat sam na seba a zaroven ze uz je spracovany
				m_ver.find(b)->second = a;



			}
		}
	}



	//vytvorenie novych primitivsets
	ps_vec.clear();
	unsigned int to = in_draw->asGeometry()->getNumPrimitiveSets();
	for (unsigned int ii=0; ii<to; ii++) {			
		in_draw->asGeometry()->setPrimitiveSet(ii, ReplaceIndices(in_draw->asGeometry()->getPrimitiveSet(ii), &m_ver));			
	}

		
}

void MyVertexRepair::apply( osg::Geode& in_geode )
{

	for (unsigned int t=0; t<in_geode.getNumDrawables(); t++) {
		DoDrawable(in_geode.getDrawable(t));
	}

	traverse( in_geode );
}